/*     */ package com.google.code.yanf4j.util;
/*     */ 
/*     */ import java.io.IOException;
/*     */ import java.nio.channels.Selector;
/*     */ import java.util.EmptyStackException;
/*     */ import java.util.Stack;
/*     */ import org.slf4j.Logger;
/*     */ import org.slf4j.LoggerFactory;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SelectorFactory
/*     */ {
/*     */   public static final int DEFAULT_MAX_SELECTORS = 20;
/*  44 */   private static final Logger logger = LoggerFactory.getLogger(SelectorFactory.class);
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final long timeout = 5000L;
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private static int maxSelectors;
/*     */   
/*     */ 
/*     */ 
/*  58 */   private static final Stack<Selector> selectors = new Stack();
/*     */   
/*     */ 
/*     */   static
/*     */   {
/*     */     try
/*     */     {
/*  65 */       setMaxSelectors(20);
/*     */     }
/*     */     catch (IOException ex) {
/*  68 */       logger.warn("SelectorFactory initializing Selector pool", ex);
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final void setMaxSelectors(int size)
/*     */     throws IOException
/*     */   {
/*  80 */     synchronized (selectors) {
/*  81 */       if (size < maxSelectors) {
/*  82 */         reduce(size);
/*     */       }
/*  84 */       else if (size > maxSelectors) {
/*  85 */         grow(size);
/*     */       }
/*     */       
/*  88 */       maxSelectors = size;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final int getMaxSelectors()
/*     */   {
/*  99 */     return maxSelectors;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final Selector getSelector()
/*     */   {
/* 109 */     synchronized (selectors) {
/* 110 */       Selector s = null;
/*     */       try {
/* 112 */         if (selectors.size() != 0) {
/* 113 */           s = (Selector)selectors.pop();
/*     */         }
/*     */       }
/*     */       catch (EmptyStackException ex) {}
/*     */       
/*     */ 
/* 119 */       int attempts = 0;
/*     */       try {
/* 121 */         while ((s == null) && (attempts < 2)) {
/* 122 */           selectors.wait(5000L);
/*     */           try {
/* 124 */             if (selectors.size() != 0) {
/* 125 */               s = (Selector)selectors.pop();
/*     */             }
/*     */           }
/*     */           catch (EmptyStackException ex) {
/*     */             break;
/*     */           }
/* 131 */           attempts++;
/*     */         }
/*     */       }
/*     */       catch (InterruptedException ex) {}
/*     */       
/* 136 */       return s;
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public static final void returnSelector(Selector s)
/*     */   {
/* 148 */     synchronized (selectors) {
/* 149 */       selectors.push(s);
/* 150 */       if (selectors.size() == 1) {
/* 151 */         selectors.notify();
/*     */       }
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */   private static void grow(int size)
/*     */     throws IOException
/*     */   {
/* 161 */     for (int i = 0; i < size - maxSelectors; i++) {
/* 162 */       selectors.add(Selector.open());
/*     */     }
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   private static void reduce(int size)
/*     */   {
/* 171 */     for (int i = 0; i < maxSelectors - size; i++) {
/*     */       try {
/* 173 */         Selector selector = (Selector)selectors.pop();
/* 174 */         selector.close();
/*     */       }
/*     */       catch (IOException e) {
/* 177 */         logger.error("SelectorFactory.reduce", e);
/*     */       }
/*     */     }
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\google\code\yanf4j\util\SelectorFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */