/*     */ package com.netapp.asup.cache;
/*     */ 
/*     */ import com.googlecode.concurrentlinkedhashmap.EvictionListener;
/*     */ import com.netapp.asup.cache.LRU.AsupLRUCache;
/*     */ import com.netapp.asup.cache.LRU.AsupLRUCacheImpl;
/*     */ import com.netapp.asup.cache.server.AsupGenericCache;
/*     */ import com.netapp.asup.cache.server.AsupServerCacheImpl;
/*     */ import com.netapp.asup.common.AsupException;
/*     */ import com.netapp.asup.common.AsupLogger;
/*     */ import com.netapp.asup.config.Configuration;
/*     */ import com.netapp.asup.exception.ARMAException;
/*     */ import java.io.File;
/*     */ import java.io.FileInputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ import java.net.InetSocketAddress;
/*     */ import java.util.Map;
/*     */ import java.util.Properties;
/*     */ import java.util.concurrent.TimeoutException;
/*     */ import net.rubyeye.xmemcached.exception.MemcachedException;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class ARMAGenericCacheImpl
/*     */   implements AsupGenericCache<String, Object>
/*     */ {
/*     */   private static final String FALSE = "false";
/*     */   private static final String ARMA_REST_PROPERTIES = "/arma_rest.properties";
/*     */   private static final double DEFAULT_MAX_AFTER_EVICTION_PERCENT = 1.0D;
/*     */   private static final double DEFAULT_MAX_THRESHOLD_PERCENT = 1.0D;
/*     */   private static final int DEFAULT_CACHE_SIZE = 10000;
/*     */   private static final String ARMA_REST_CLIENT_CACHE_MAX_AFTER_EVICTION_PERCENTAGE = "arma.rest.client.cache.max.afterEviction.percentage";
/*     */   private static final String ARMA_REST_CLIENT_CACHE_MAX_THRESHOLD_PERCENTAGE = "arma.rest.client.cache.max.threshold.percentage";
/*     */   private static final String ARMA_REST_CLIENT_CACHE_MAX_SIZE = "arma.rest.client.cache.max.size";
/*     */   private static final String ARMA_REST_CLIENT_CACHE_WEIGHT_TYPE = "arma.rest.client.cache.weight.type";
/*  46 */   private static final AsupLogger logger = AsupLogger.getLogger(ARMAGenericCacheImpl.class);
/*     */   
/*  48 */   AsupGenericCache<String, Object> serverCache = null;
/*  49 */   AsupLRUCache<String, Object> clientCache = null;
/*  50 */   private boolean isMemcachePropertyFileExternalized = false;
/*     */   
/*  52 */   private static String CLIENT_CACHE_ENABLED = "arma.rest.client.cache.enabled";
/*     */   
/*  54 */   private static String SERVER_CACHE_ENABLED = "arma.rest.server.cache.enabled";
/*     */   
/*     */   private static final String MEMCACHE_CONNECTIONS = "arma.rest.memcache.node.hostnameandport";
/*     */   
/*     */   private static final int DEFAULT_CACHE_EXPIRY = 86400;
/*     */   private String memcacheConnections;
/*     */   private InputStream is;
/*     */   
/*     */   public ARMAGenericCacheImpl(Configuration config)
/*     */     throws ARMAException, IOException
/*     */   {
/*  65 */     String isClientCacheEnabled = config.getString(CLIENT_CACHE_ENABLED);
/*  66 */     String isServerCacheEnabled = config.getString(SERVER_CACHE_ENABLED);
/*     */     
/*  68 */     String externalPropertyFSLocation = System.getProperty("ext.prop.dir");
/*     */     
/*  70 */     File memcachePropertyFile = new File(externalPropertyFSLocation + "/arma_rest.properties");
/*     */     
/*  72 */     if (memcachePropertyFile.exists()) {
/*  73 */       this.is = new FileInputStream(memcachePropertyFile);
/*  74 */       this.isMemcachePropertyFileExternalized = true;
/*  75 */       logger.info(null, "reading memcache properties from a externalized property file..");
/*     */     }
/*     */     else {
/*  78 */       this.is = getClass().getResourceAsStream("/arma_rest.properties");
/*  79 */       logger.info(null, "reading memcache property from the classpath..");
/*     */     }
/*     */     
/*  82 */     if (this.is != null) {
/*  83 */       Properties p = new Properties();
/*  84 */       p.load(this.is);
/*  85 */       this.memcacheConnections = p.getProperty("arma.rest.memcache.node.hostnameandport");
/*  86 */       if (logger.isDebugEnabled()) {
/*  87 */         logger.debug("-9999", "Memcache connections, hostname and port property : " + this.memcacheConnections);
/*     */       }
/*     */     }
/*     */     else
/*     */     {
/*  92 */       logger.warn("-9999", "MemCache is disabled .Since the following file is not found in classpath:/arma_rest.properties");
/*     */       
/*     */ 
/*  95 */       isServerCacheEnabled = "false";
/*     */     }
/*     */     
/*  98 */     if (Boolean.parseBoolean(isServerCacheEnabled)) {
/*     */       try
/*     */       {
/* 101 */         this.serverCache = new AsupServerCacheImpl(this.memcacheConnections);
/* 102 */         logger.info("-9999", "Initialized Mem cache: server details{" + this.memcacheConnections + "}");
/*     */       } catch (AsupException e) {
/* 104 */         throw new ARMAException(logger, "-9999", e.getMessage());
/*     */       }
/*     */     }
/*     */     
/* 108 */     if (Boolean.parseBoolean(isClientCacheEnabled))
/*     */     {
/* 110 */       String weightType = config.getString("arma.rest.client.cache.weight.type");
/* 111 */       int cacheSize = config.getInt("arma.rest.client.cache.max.size", 10000);
/* 112 */       double maxThresholdPercent = config.getDouble("arma.rest.client.cache.max.threshold.percentage", 1.0D);
/* 113 */       double afterEvictThresholdPercent = config.getDouble("arma.rest.client.cache.max.afterEviction.percentage", 1.0D);
/* 114 */       AsupLRUCacheImpl.initialize(weightType, cacheSize, maxThresholdPercent, afterEvictThresholdPercent);
/* 115 */       this.clientCache = AsupLRUCacheImpl.getinstance();
/* 116 */       logger.info("-9999", String.format("Initialized in-memory cache: Weight type: %s, Size: %d", new Object[] { weightType, Integer.valueOf(cacheSize) }));
/*     */     }
/*     */   }
/*     */   
/*     */   public boolean isMemcachePropertyFileExternalized() {
/* 121 */     return this.isMemcachePropertyFileExternalized;
/*     */   }
/*     */   
/*     */   public Object get(String key)
/*     */   {
/* 126 */     Object value = null;
/* 127 */     if (this.clientCache != null) {
/* 128 */       if (logger.isDebugEnabled()) {
/* 129 */         logger.debug("-9999", "Fetching from in-memory cache for hash:" + key);
/*     */       }
/*     */       
/* 132 */       value = this.clientCache.get(key);
/*     */     }
/*     */     
/* 135 */     if ((value == null) && (this.serverCache != null)) {
/* 136 */       if (logger.isDebugEnabled()) {
/* 137 */         logger.debug("-9999", "Not present in in-memory cache.Fetching from mem-cache cache for hash:" + key);
/*     */       }
/* 139 */       value = this.serverCache.get(key);
/*     */     }
/* 141 */     return value;
/*     */   }
/*     */   
/*     */   public Object putIfAbsent(String key, Object value)
/*     */   {
/* 146 */     Object oldValue = null;
/* 147 */     if (this.clientCache != null) {
/* 148 */       oldValue = this.clientCache.putIfAbsent(key, value);
/*     */     }
/* 150 */     if (this.serverCache != null) {
/*     */       try {
/* 152 */         oldValue = this.serverCache.putIfAbsent(key, 86400, value);
/*     */       } catch (Exception e) {
/* 154 */         logger.warn("-9999", "Unable to put into cache :" + key, e);
/*     */       }
/*     */     }
/*     */     
/* 158 */     return oldValue;
/*     */   }
/*     */   
/*     */   public Object put(String key, Object value)
/*     */   {
/* 163 */     Object oldValue = null;
/* 164 */     if (this.clientCache != null) {
/* 165 */       oldValue = this.clientCache.put(key, value);
/*     */     }
/*     */     
/* 168 */     if (this.serverCache != null) {
/*     */       try {
/* 170 */         oldValue = this.serverCache.put(key, 86400, value);
/*     */       } catch (Exception e) {
/* 172 */         logger.warn("-9999", "Unable to put into cache :" + key, e);
/*     */       }
/*     */     }
/* 175 */     return oldValue;
/*     */   }
/*     */   
/*     */   public Object remove(String key)
/*     */   {
/* 180 */     Object oldValue = null;
/* 181 */     if (this.clientCache != null) {
/* 182 */       oldValue = this.clientCache.remove(key);
/*     */     }
/* 184 */     if (this.serverCache != null) {
/* 185 */       oldValue = this.serverCache.remove(key);
/*     */     }
/* 187 */     return oldValue;
/*     */   }
/*     */   
/*     */   public void clear()
/*     */   {
/* 192 */     if (this.clientCache != null) {
/* 193 */       this.clientCache.clear();
/*     */     }
/* 195 */     if (this.serverCache != null) {
/* 196 */       this.serverCache.clear();
/*     */     }
/*     */   }
/*     */   
/*     */   public void shutdown()
/*     */   {
/* 202 */     if (this.clientCache != null) {
/* 203 */       this.clientCache.shutdown();
/*     */     }
/* 205 */     if (this.serverCache != null) {
/* 206 */       this.serverCache.shutdown();
/*     */     }
/*     */   }
/*     */   
/*     */   public String getCacheStats()
/*     */   {
/* 212 */     StringBuilder builder = new StringBuilder();
/* 213 */     if (this.clientCache != null)
/* 214 */       builder.append("LRU Cache Stats : " + this.clientCache.getCacheStats());
/* 215 */     if (this.serverCache != null) {
/* 216 */       builder.append("\nMemcache Stats :" + this.serverCache.getCacheStats());
/*     */     }
/* 218 */     return builder.toString();
/*     */   }
/*     */   
/*     */   public void removeServer(String serverConnection) throws AsupException, IOException
/*     */   {
/* 223 */     if (this.serverCache != null) {
/* 224 */       this.serverCache.removeServer(serverConnection);
/*     */     } else {
/* 226 */       logger.warn(null, "Server cache is disabled, so can't remove any memcache servers..");
/*     */     }
/*     */   }
/*     */   
/*     */   public void addEvictionListener(EvictionListener<String, Object> listner)
/*     */   {
/* 232 */     this.clientCache.addEvictionListener(listner);
/*     */   }
/*     */   
/*     */   public void addServer(String serverConnection) throws AsupException, IOException
/*     */   {
/* 237 */     if (this.serverCache != null) { this.serverCache.addServer(serverConnection);
/*     */     }
/*     */   }
/*     */   
/*     */   public <T> T getAndTouch(String key, int ttl)
/*     */   {
/* 243 */     if (this.serverCache != null) return (T)this.serverCache.getAndTouch(key, ttl); return null;
/*     */   }
/*     */   
/*     */   public <T> Map<String, T> getBulk(String... keys)
/*     */     throws TimeoutException, InterruptedException, MemcachedException
/*     */   {
/* 249 */     if (this.serverCache != null) return this.serverCache.getBulk(keys); return null;
/*     */   }
/*     */   
/*     */ 
/*     */   public Map<InetSocketAddress, Map<String, String>> getStats()
/*     */   {
/* 255 */     if (this.serverCache != null) return this.serverCache.getStats(); return null;
/*     */   }
/*     */   
/*     */   public Object put(String key, int expiry, Object value)
/*     */     throws TimeoutException, InterruptedException, MemcachedException
/*     */   {
/* 261 */     if (this.serverCache != null) return this.serverCache.put(key, expiry, value); return null;
/*     */   }
/*     */   
/*     */   public Object putIfAbsent(String key, int expiry, Object value)
/*     */     throws TimeoutException, InterruptedException, MemcachedException
/*     */   {
/* 267 */     if (this.serverCache != null) return this.serverCache.putIfAbsent(key, expiry, value); return null;
/*     */   }
/*     */   
/*     */   public boolean replace(String key, int ttl, Object value)
/*     */   {
/* 272 */     if (this.serverCache != null) return this.serverCache.replace(key, ttl, value);
/* 273 */     return false;
/*     */   }
/*     */   
/*     */   public boolean touch(String key, int ttl)
/*     */   {
/* 278 */     if (this.serverCache != null) return this.serverCache.touch(key, ttl); return false;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\cache\ARMAGenericCacheImpl.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */