/*    */ package com.netapp.asup.data.objects;
/*    */ 
/*    */ import com.netapp.asup.client.arma.data.ARMAObjectInstance;
/*    */ import com.netapp.asup.common.AsupLogger;
/*    */ import com.netapp.asup.data.function.RegisteredFunction;
/*    */ import com.netapp.asup.exception.ARMAFunctionExecutionException;
/*    */ import com.netapp.asupdata.client.ARMA.utils.IdentifierUtils;
/*    */ import java.io.IOException;
/*    */ import java.nio.ByteBuffer;
/*    */ import org.apache.avro.generic.GenericData.Record;
/*    */ import org.apache.avro.generic.GenericRecord;
/*    */ import org.apache.avro.util.Utf8;
/*    */ 
/*    */ 
/*    */ public class FunctionRequestData<I, O>
/*    */   extends ByteWrappedGenericRecord<GenericRecord, GenericRecord>
/*    */ {
/* 18 */   protected static final AsupLogger LOG = AsupLogger.getLogger(FunctionRequestData.class);
/*    */   
/*    */   private static final String OBJECT_NAME = "objectName";
/*    */   
/*    */   private static final String FUNCTION_NAME = "functionName";
/*    */   
/*    */   private static final String INPUT_VALUE = "inputValue";
/*    */   private static final String FUNC_ID = "funcId";
/*    */   private final RegisteredFunction<I, O> registeredFunction;
/*    */   private final GenericRecord input;
/*    */   
/*    */   public FunctionRequestData(String id, String objectName, String identifier, RegisteredFunction<I, O> registeredFunction, GenericRecord input)
/*    */   {
/* 31 */     super(ARMASchema.FUNCTION_REQUEST_DATA, input == null ? null : input.getSchema(), id);
/* 32 */     this.object = new GenericData.Record(ARMASchema.FUNCTION_REQUEST_DATA.getSchema());
/* 33 */     String uniqueFunctionIdentifier = IdentifierUtils.getUniqueFunctionIdentifier(objectName, identifier, registeredFunction.getFunctionName());
/* 34 */     AvroGenricRecordUtility.putStringValue((GenericRecord)this.object, "funcId", uniqueFunctionIdentifier);
/* 35 */     AvroGenricRecordUtility.putStringValue((GenericRecord)this.object, "functionName", registeredFunction.getFunctionName());
/* 36 */     AvroGenricRecordUtility.putStringValue((GenericRecord)this.object, "objectName", objectName);
/* 37 */     this.input = input;
/* 38 */     this.registeredFunction = registeredFunction;
/*    */   }
/*    */   
/*    */   public FunctionRequestData(String id, GenericRecord object, RegisteredFunction<I, O> registeredFunction) throws IOException {
/* 42 */     super(ARMASchema.FUNCTION_REQUEST_DATA, registeredFunction.getInputSchema(), id);
/* 43 */     this.object = object;
/* 44 */     if (registeredFunction.getInputSchema() == null) this.input = null; else
/* 45 */       this.input = ((GenericRecord)deserializeData(((ByteBuffer)object.get("inputValue")).array()));
/* 46 */     this.registeredFunction = registeredFunction;
/*    */   }
/*    */   
/*    */   public String getFunctionName() {
/* 50 */     return AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "functionName");
/*    */   }
/*    */   
/*    */   public String getObjectName()
/*    */   {
/* 55 */     return AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "objectName");
/*    */   }
/*    */   
/*    */   public Utf8 getFunctionId() {
/* 59 */     return AvroGenricRecordUtility.getUtf8Value((GenericRecord)this.object, "funcId");
/*    */   }
/*    */   
/*    */   public GenericRecord getObject() throws IOException
/*    */   {
/* 64 */     ((GenericRecord)this.object).put("inputValue", ByteBuffer.wrap(serializeData(this.input)));
/* 65 */     return (GenericRecord)this.object;
/*    */   }
/*    */   
/*    */ 
/*    */   public String toString()
/*    */   {
/* 71 */     StringBuilder funcStr = new StringBuilder(123);
/* 72 */     funcStr.append("Function Object Name:");
/* 73 */     funcStr.append(AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "objectName"));
/* 74 */     funcStr.append(",Function Identifier:");
/* 75 */     funcStr.append(AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "funcId"));
/* 76 */     funcStr.append(",Function Name:");
/* 77 */     funcStr.append(AvroGenricRecordUtility.getStringValue((GenericRecord)this.object, "functionName"));
/* 78 */     funcStr.append('}');
/* 79 */     return funcStr.toString();
/*    */   }
/*    */   
/*    */   public GenericRecord execute(ARMAObjectInstance instance) throws ARMAFunctionExecutionException
/*    */   {
/*    */     try {
/* 85 */       return instance.executeGenericIOFunction(getRegisteredFunction(), this.input);
/*    */     } catch (Exception e) {
/* 87 */       throw new ARMAFunctionExecutionException(LOG, this.id, "Unknown exception caught in executing function :" + toString() + " caused by :" + e.getMessage(), e);
/*    */     }
/*    */   }
/*    */   
/*    */   public RegisteredFunction<I, O> getRegisteredFunction() {
/* 92 */     return this.registeredFunction;
/*    */   }
/*    */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\com\netapp\asup\data\objects\FunctionRequestData.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */