/*     */ package org.apache.avro.data;
/*     */ 
/*     */ import java.io.ByteArrayOutputStream;
/*     */ import java.io.IOException;
/*     */ import java.util.Arrays;
/*     */ import java.util.List;
/*     */ import java.util.concurrent.ConcurrentHashMap;
/*     */ import java.util.concurrent.ConcurrentMap;
/*     */ import org.apache.avro.AvroRuntimeException;
/*     */ import org.apache.avro.Schema;
/*     */ import org.apache.avro.Schema.Field;
/*     */ import org.apache.avro.Schema.Type;
/*     */ import org.apache.avro.generic.GenericData;
/*     */ import org.apache.avro.generic.IndexedRecord;
/*     */ import org.apache.avro.io.BinaryDecoder;
/*     */ import org.apache.avro.io.BinaryEncoder;
/*     */ import org.apache.avro.io.DatumReader;
/*     */ import org.apache.avro.io.DecoderFactory;
/*     */ import org.apache.avro.io.EncoderFactory;
/*     */ import org.apache.avro.io.parsing.ResolvingGrammarGenerator;
/*     */ import org.codehaus.jackson.JsonNode;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public abstract class RecordBuilderBase<T extends IndexedRecord>
/*     */   implements RecordBuilder<T>
/*     */ {
/*  43 */   private static final ConcurrentMap<String, ConcurrentMap<Integer, Object>> DEFAULT_VALUE_CACHE = new ConcurrentHashMap();
/*     */   
/*  45 */   private static final Schema.Field[] EMPTY_FIELDS = new Schema.Field[0];
/*     */   private final Schema schema;
/*     */   private final Schema.Field[] fields;
/*     */   private final boolean[] fieldSetFlags;
/*     */   private final GenericData data;
/*  50 */   private BinaryEncoder encoder = null;
/*  51 */   private BinaryDecoder decoder = null;
/*     */   
/*  53 */   protected final Schema schema() { return this.schema; }
/*  54 */   protected final Schema.Field[] fields() { return this.fields; }
/*  55 */   protected final boolean[] fieldSetFlags() { return this.fieldSetFlags; }
/*  56 */   protected final GenericData data() { return this.data; }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */   protected RecordBuilderBase(Schema schema, GenericData data)
/*     */   {
/*  63 */     this.schema = schema;
/*  64 */     this.data = data;
/*  65 */     this.fields = ((Schema.Field[])schema.getFields().toArray(EMPTY_FIELDS));
/*  66 */     this.fieldSetFlags = new boolean[this.fields.length];
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected RecordBuilderBase(RecordBuilderBase<T> other, GenericData data)
/*     */   {
/*  75 */     this.schema = other.schema;
/*  76 */     this.data = data;
/*  77 */     this.fields = ((Schema.Field[])this.schema.getFields().toArray(EMPTY_FIELDS));
/*  78 */     this.fieldSetFlags = new boolean[other.fieldSetFlags.length];
/*  79 */     System.arraycopy(other.fieldSetFlags, 0, this.fieldSetFlags, 0, this.fieldSetFlags.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected void validate(Schema.Field field, Object value)
/*     */   {
/*  96 */     if (isValidValue(field, value)) {
/*  97 */       return;
/*     */     }
/*  99 */     if (field.defaultValue() != null) {
/* 100 */       return;
/*     */     }
/*     */     
/* 103 */     throw new AvroRuntimeException("Field " + field + " does not accept null values");
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected static boolean isValidValue(Schema.Field f, Object value)
/*     */   {
/* 115 */     if (value != null) {
/* 116 */       return true;
/*     */     }
/*     */     
/* 119 */     Schema schema = f.schema();
/* 120 */     Schema.Type type = schema.getType();
/*     */     
/*     */ 
/* 123 */     if (type == Schema.Type.NULL) {
/* 124 */       return true;
/*     */     }
/*     */     
/*     */ 
/* 128 */     if (type == Schema.Type.UNION) {
/* 129 */       for (Schema s : schema.getTypes()) {
/* 130 */         if (s.getType() == Schema.Type.NULL) {
/* 131 */           return true;
/*     */         }
/*     */       }
/*     */     }
/*     */     
/*     */ 
/* 137 */     return false;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   protected Object defaultValue(Schema.Field field)
/*     */     throws IOException
/*     */   {
/* 149 */     JsonNode defaultJsonValue = field.defaultValue();
/* 150 */     if (defaultJsonValue == null) {
/* 151 */       throw new AvroRuntimeException("Field " + field + " not set and has no default value");
/*     */     }
/* 153 */     if ((defaultJsonValue.isNull()) && ((field.schema().getType() == Schema.Type.NULL) || ((field.schema().getType() == Schema.Type.UNION) && (((Schema)field.schema().getTypes().get(0)).getType() == Schema.Type.NULL))))
/*     */     {
/*     */ 
/*     */ 
/* 157 */       return null;
/*     */     }
/*     */     
/*     */ 
/* 161 */     Object defaultValue = null;
/*     */     
/*     */ 
/* 164 */     ConcurrentMap<Integer, Object> defaultSchemaValues = (ConcurrentMap)DEFAULT_VALUE_CACHE.get(this.schema.getFullName());
/*     */     
/* 166 */     if (defaultSchemaValues == null) {
/* 167 */       DEFAULT_VALUE_CACHE.putIfAbsent(this.schema.getFullName(), new ConcurrentHashMap(this.fields.length));
/*     */       
/* 169 */       defaultSchemaValues = (ConcurrentMap)DEFAULT_VALUE_CACHE.get(this.schema.getFullName());
/*     */     }
/* 171 */     defaultValue = defaultSchemaValues.get(Integer.valueOf(field.pos()));
/*     */     
/*     */ 
/*     */ 
/* 175 */     if (defaultValue == null) {
/* 176 */       ByteArrayOutputStream baos = new ByteArrayOutputStream();
/* 177 */       this.encoder = EncoderFactory.get().binaryEncoder(baos, this.encoder);
/* 178 */       ResolvingGrammarGenerator.encode(this.encoder, field.schema(), defaultJsonValue);
/*     */       
/* 180 */       this.encoder.flush();
/* 181 */       this.decoder = DecoderFactory.get().binaryDecoder(baos.toByteArray(), this.decoder);
/*     */       
/* 183 */       defaultValue = this.data.createDatumReader(field.schema()).read(null, this.decoder);
/*     */       
/* 185 */       defaultSchemaValues.putIfAbsent(Integer.valueOf(field.pos()), defaultValue);
/*     */     }
/*     */     
/*     */ 
/*     */ 
/* 190 */     return this.data.deepCopy(field.schema(), defaultValue);
/*     */   }
/*     */   
/*     */   public int hashCode()
/*     */   {
/* 195 */     int prime = 31;
/* 196 */     int result = 1;
/* 197 */     result = 31 * result + Arrays.hashCode(this.fieldSetFlags);
/* 198 */     result = 31 * result + (this.schema == null ? 0 : this.schema.hashCode());
/* 199 */     return result;
/*     */   }
/*     */   
/*     */   public boolean equals(Object obj)
/*     */   {
/* 204 */     if (this == obj)
/* 205 */       return true;
/* 206 */     if (obj == null)
/* 207 */       return false;
/* 208 */     if (getClass() != obj.getClass()) {
/* 209 */       return false;
/*     */     }
/* 211 */     RecordBuilderBase other = (RecordBuilderBase)obj;
/* 212 */     if (!Arrays.equals(this.fieldSetFlags, other.fieldSetFlags))
/* 213 */       return false;
/* 214 */     if (this.schema == null) {
/* 215 */       if (other.schema != null)
/* 216 */         return false;
/* 217 */     } else if (!this.schema.equals(other.schema))
/* 218 */       return false;
/* 219 */     return true;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\apache\avro\data\RecordBuilderBase.class
 * Java compiler version: 6 (50.0)
 * JD-Core Version:       0.7.1
 */