/*      */ package org.codehaus.jackson.map.deser;
/*      */ 
/*      */ import java.util.HashSet;
/*      */ import java.util.Map;
/*      */ import java.util.Map.Entry;
/*      */ import org.codehaus.jackson.map.AbstractTypeResolver;
/*      */ import org.codehaus.jackson.map.AnnotationIntrospector;
/*      */ import org.codehaus.jackson.map.AnnotationIntrospector.ReferenceProperty;
/*      */ import org.codehaus.jackson.map.BeanProperty;
/*      */ import org.codehaus.jackson.map.BeanProperty.Std;
/*      */ import org.codehaus.jackson.map.DeserializationConfig;
/*      */ import org.codehaus.jackson.map.DeserializationConfig.Feature;
/*      */ import org.codehaus.jackson.map.DeserializerFactory.Config;
/*      */ import org.codehaus.jackson.map.DeserializerProvider;
/*      */ import org.codehaus.jackson.map.Deserializers;
/*      */ import org.codehaus.jackson.map.JsonDeserializer;
/*      */ import org.codehaus.jackson.map.JsonMappingException;
/*      */ import org.codehaus.jackson.map.KeyDeserializers;
/*      */ import org.codehaus.jackson.map.TypeDeserializer;
/*      */ import org.codehaus.jackson.map.introspect.AnnotatedField;
/*      */ import org.codehaus.jackson.map.introspect.AnnotatedMethod;
/*      */ import org.codehaus.jackson.map.introspect.BasicBeanDescription;
/*      */ import org.codehaus.jackson.map.introspect.VisibilityChecker;
/*      */ import org.codehaus.jackson.map.util.ArrayBuilders;
/*      */ import org.codehaus.jackson.type.JavaType;
/*      */ 
/*      */ public class BeanDeserializerFactory extends BasicDeserializerFactory
/*      */ {
/*   29 */   private static final Class<?>[] INIT_CAUSE_PARAMS = { Throwable.class };
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public static class ConfigImpl
/*      */     extends DeserializerFactory.Config
/*      */   {
/*   44 */     protected static final KeyDeserializers[] NO_KEY_DESERIALIZERS = new KeyDeserializers[0];
/*   45 */     protected static final BeanDeserializerModifier[] NO_MODIFIERS = new BeanDeserializerModifier[0];
/*   46 */     protected static final AbstractTypeResolver[] NO_ABSTRACT_TYPE_RESOLVERS = new AbstractTypeResolver[0];
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected final Deserializers[] _additionalDeserializers;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected final KeyDeserializers[] _additionalKeyDeserializers;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected final BeanDeserializerModifier[] _modifiers;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected final AbstractTypeResolver[] _abstractTypeResolvers;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     public ConfigImpl()
/*      */     {
/*   84 */       this(null, null, null, null);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */     protected ConfigImpl(Deserializers[] allAdditionalDeserializers, KeyDeserializers[] allAdditionalKeyDeserializers, BeanDeserializerModifier[] modifiers, AbstractTypeResolver[] atr)
/*      */     {
/*   96 */       this._additionalDeserializers = (allAdditionalDeserializers == null ? BeanDeserializerFactory.NO_DESERIALIZERS : allAdditionalDeserializers);
/*      */       
/*   98 */       this._additionalKeyDeserializers = (allAdditionalKeyDeserializers == null ? NO_KEY_DESERIALIZERS : allAdditionalKeyDeserializers);
/*      */       
/*  100 */       this._modifiers = (modifiers == null ? NO_MODIFIERS : modifiers);
/*  101 */       this._abstractTypeResolvers = (atr == null ? NO_ABSTRACT_TYPE_RESOLVERS : atr);
/*      */     }
/*      */     
/*      */ 
/*      */     public DeserializerFactory.Config withAdditionalDeserializers(Deserializers additional)
/*      */     {
/*  107 */       if (additional == null) {
/*  108 */         throw new IllegalArgumentException("Can not pass null Deserializers");
/*      */       }
/*  110 */       Deserializers[] all = (Deserializers[])ArrayBuilders.insertInListNoDup(this._additionalDeserializers, additional);
/*  111 */       return new ConfigImpl(all, this._additionalKeyDeserializers, this._modifiers, this._abstractTypeResolvers);
/*      */     }
/*      */     
/*      */ 
/*      */     public DeserializerFactory.Config withAdditionalKeyDeserializers(KeyDeserializers additional)
/*      */     {
/*  117 */       if (additional == null) {
/*  118 */         throw new IllegalArgumentException("Can not pass null KeyDeserializers");
/*      */       }
/*  120 */       KeyDeserializers[] all = (KeyDeserializers[])ArrayBuilders.insertInListNoDup(this._additionalKeyDeserializers, additional);
/*  121 */       return new ConfigImpl(this._additionalDeserializers, all, this._modifiers, this._abstractTypeResolvers);
/*      */     }
/*      */     
/*      */ 
/*      */     public DeserializerFactory.Config withDeserializerModifier(BeanDeserializerModifier modifier)
/*      */     {
/*  127 */       if (modifier == null) {
/*  128 */         throw new IllegalArgumentException("Can not pass null modifier");
/*      */       }
/*  130 */       BeanDeserializerModifier[] all = (BeanDeserializerModifier[])ArrayBuilders.insertInListNoDup(this._modifiers, modifier);
/*  131 */       return new ConfigImpl(this._additionalDeserializers, this._additionalKeyDeserializers, all, this._abstractTypeResolvers);
/*      */     }
/*      */     
/*      */ 
/*      */     public DeserializerFactory.Config withAbstractTypeResolver(AbstractTypeResolver resolver)
/*      */     {
/*  137 */       if (resolver == null) {
/*  138 */         throw new IllegalArgumentException("Can not pass null resolver");
/*      */       }
/*  140 */       AbstractTypeResolver[] all = (AbstractTypeResolver[])ArrayBuilders.insertInListNoDup(this._abstractTypeResolvers, resolver);
/*  141 */       return new ConfigImpl(this._additionalDeserializers, this._additionalKeyDeserializers, this._modifiers, all);
/*      */     }
/*      */     
/*      */     public boolean hasDeserializers() {
/*  145 */       return this._additionalDeserializers.length > 0;
/*      */     }
/*      */     
/*  148 */     public boolean hasKeyDeserializers() { return this._additionalKeyDeserializers.length > 0; }
/*      */     
/*      */     public boolean hasDeserializerModifiers() {
/*  151 */       return this._modifiers.length > 0;
/*      */     }
/*      */     
/*  154 */     public boolean hasAbstractTypeResolvers() { return this._abstractTypeResolvers.length > 0; }
/*      */     
/*      */     public Iterable<Deserializers> deserializers()
/*      */     {
/*  158 */       return ArrayBuilders.arrayAsIterable(this._additionalDeserializers);
/*      */     }
/*      */     
/*      */     public Iterable<KeyDeserializers> keyDeserializers()
/*      */     {
/*  163 */       return ArrayBuilders.arrayAsIterable(this._additionalKeyDeserializers);
/*      */     }
/*      */     
/*      */     public Iterable<BeanDeserializerModifier> deserializerModifiers()
/*      */     {
/*  168 */       return ArrayBuilders.arrayAsIterable(this._modifiers);
/*      */     }
/*      */     
/*      */     public Iterable<AbstractTypeResolver> abstractTypeResolvers()
/*      */     {
/*  173 */       return ArrayBuilders.arrayAsIterable(this._abstractTypeResolvers);
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  187 */   public static final BeanDeserializerFactory instance = new BeanDeserializerFactory(null);
/*      */   
/*      */ 
/*      */ 
/*      */   protected final DeserializerFactory.Config _factoryConfig;
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   @Deprecated
/*      */   public BeanDeserializerFactory()
/*      */   {
/*  199 */     this(null);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   public BeanDeserializerFactory(DeserializerFactory.Config config)
/*      */   {
/*  206 */     if (config == null) {
/*  207 */       config = new ConfigImpl();
/*      */     }
/*  209 */     this._factoryConfig = config;
/*      */   }
/*      */   
/*      */   public final DeserializerFactory.Config getConfig()
/*      */   {
/*  214 */     return this._factoryConfig;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public org.codehaus.jackson.map.DeserializerFactory withConfig(DeserializerFactory.Config config)
/*      */   {
/*  227 */     if (this._factoryConfig == config) {
/*  228 */       return this;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  237 */     if (getClass() != BeanDeserializerFactory.class) {
/*  238 */       throw new IllegalStateException("Subtype of BeanDeserializerFactory (" + getClass().getName() + ") has not properly overridden method 'withAdditionalDeserializers': can not instantiate subtype with " + "additional deserializer definitions");
/*      */     }
/*      */     
/*      */ 
/*  242 */     return new BeanDeserializerFactory(config);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public org.codehaus.jackson.map.KeyDeserializer createKeyDeserializer(DeserializationConfig config, JavaType type, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*      */     BasicBeanDescription beanDesc;
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  257 */     if (this._factoryConfig.hasKeyDeserializers()) {
/*  258 */       beanDesc = (BasicBeanDescription)config.introspectClassAnnotations(type.getRawClass());
/*  259 */       for (KeyDeserializers d : this._factoryConfig.keyDeserializers()) {
/*  260 */         org.codehaus.jackson.map.KeyDeserializer deser = d.findKeyDeserializer(type, config, beanDesc, property);
/*  261 */         if (deser != null) {
/*  262 */           return deser;
/*      */         }
/*      */       }
/*      */     }
/*  266 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomArrayDeserializer(org.codehaus.jackson.map.type.ArrayType type, DeserializationConfig config, DeserializerProvider provider, BeanProperty property, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
/*      */     throws JsonMappingException
/*      */   {
/*  276 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  277 */       JsonDeserializer<?> deser = d.findArrayDeserializer(type, config, provider, property, elementTypeDeserializer, elementDeserializer);
/*      */       
/*  279 */       if (deser != null) {
/*  280 */         return deser;
/*      */       }
/*      */     }
/*  283 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomCollectionDeserializer(org.codehaus.jackson.map.type.CollectionType type, DeserializationConfig config, DeserializerProvider provider, BasicBeanDescription beanDesc, BeanProperty property, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
/*      */     throws JsonMappingException
/*      */   {
/*  293 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  294 */       JsonDeserializer<?> deser = d.findCollectionDeserializer(type, config, provider, beanDesc, property, elementTypeDeserializer, elementDeserializer);
/*      */       
/*  296 */       if (deser != null) {
/*  297 */         return deser;
/*      */       }
/*      */     }
/*  300 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomCollectionLikeDeserializer(org.codehaus.jackson.map.type.CollectionLikeType type, DeserializationConfig config, DeserializerProvider provider, BasicBeanDescription beanDesc, BeanProperty property, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
/*      */     throws JsonMappingException
/*      */   {
/*  310 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  311 */       JsonDeserializer<?> deser = d.findCollectionLikeDeserializer(type, config, provider, beanDesc, property, elementTypeDeserializer, elementDeserializer);
/*      */       
/*  313 */       if (deser != null) {
/*  314 */         return deser;
/*      */       }
/*      */     }
/*  317 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomEnumDeserializer(Class<?> type, DeserializationConfig config, BasicBeanDescription beanDesc, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  325 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  326 */       JsonDeserializer<?> deser = d.findEnumDeserializer(type, config, beanDesc, property);
/*  327 */       if (deser != null) {
/*  328 */         return deser;
/*      */       }
/*      */     }
/*  331 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomMapDeserializer(org.codehaus.jackson.map.type.MapType type, DeserializationConfig config, DeserializerProvider provider, BasicBeanDescription beanDesc, BeanProperty property, org.codehaus.jackson.map.KeyDeserializer keyDeserializer, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
/*      */     throws JsonMappingException
/*      */   {
/*  342 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  343 */       JsonDeserializer<?> deser = d.findMapDeserializer(type, config, provider, beanDesc, property, keyDeserializer, elementTypeDeserializer, elementDeserializer);
/*      */       
/*  345 */       if (deser != null) {
/*  346 */         return deser;
/*      */       }
/*      */     }
/*  349 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomMapLikeDeserializer(org.codehaus.jackson.map.type.MapLikeType type, DeserializationConfig config, DeserializerProvider provider, BasicBeanDescription beanDesc, BeanProperty property, org.codehaus.jackson.map.KeyDeserializer keyDeserializer, TypeDeserializer elementTypeDeserializer, JsonDeserializer<?> elementDeserializer)
/*      */     throws JsonMappingException
/*      */   {
/*  360 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  361 */       JsonDeserializer<?> deser = d.findMapLikeDeserializer(type, config, provider, beanDesc, property, keyDeserializer, elementTypeDeserializer, elementDeserializer);
/*      */       
/*  363 */       if (deser != null) {
/*  364 */         return deser;
/*      */       }
/*      */     }
/*  367 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<?> _findCustomTreeNodeDeserializer(Class<? extends org.codehaus.jackson.JsonNode> type, DeserializationConfig config, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  375 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  376 */       JsonDeserializer<?> deser = d.findTreeNodeDeserializer(type, config, property);
/*  377 */       if (deser != null) {
/*  378 */         return deser;
/*      */       }
/*      */     }
/*  381 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JsonDeserializer<Object> _findCustomBeanDeserializer(JavaType type, DeserializationConfig config, DeserializerProvider provider, BasicBeanDescription beanDesc, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  390 */     for (Deserializers d : this._factoryConfig.deserializers()) {
/*  391 */       JsonDeserializer<?> deser = d.findBeanDeserializer(type, config, provider, beanDesc, property);
/*  392 */       if (deser != null) {
/*  393 */         return deser;
/*      */       }
/*      */     }
/*  396 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonDeserializer<Object> createBeanDeserializer(DeserializationConfig config, DeserializerProvider p, JavaType type, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  416 */     if (type.isAbstract()) {
/*  417 */       type = mapAbstractType(config, type);
/*      */     }
/*      */     
/*      */ 
/*  421 */     BasicBeanDescription beanDesc = (BasicBeanDescription)config.introspect(type);
/*  422 */     JsonDeserializer<Object> ad = findDeserializerFromAnnotation(config, beanDesc.getClassInfo(), property);
/*  423 */     if (ad != null) {
/*  424 */       return ad;
/*      */     }
/*      */     
/*  427 */     JavaType newType = modifyTypeByAnnotation(config, beanDesc.getClassInfo(), type, null);
/*  428 */     if (newType.getRawClass() != type.getRawClass()) {
/*  429 */       type = newType;
/*  430 */       beanDesc = (BasicBeanDescription)config.introspect(type);
/*      */     }
/*      */     
/*  433 */     JsonDeserializer<Object> custom = _findCustomBeanDeserializer(type, config, p, beanDesc, property);
/*  434 */     if (custom != null) {
/*  435 */       return custom;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  441 */     if (type.isThrowable()) {
/*  442 */       return buildThrowableDeserializer(config, type, beanDesc, property);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  447 */     if (type.isAbstract())
/*      */     {
/*  449 */       JavaType concreteType = materializeAbstractType(config, beanDesc);
/*  450 */       if (concreteType != null)
/*      */       {
/*      */ 
/*      */ 
/*  454 */         beanDesc = (BasicBeanDescription)config.introspect(concreteType);
/*  455 */         return buildBeanDeserializer(config, concreteType, beanDesc, property);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  460 */     JsonDeserializer<Object> deser = findStdBeanDeserializer(config, p, type, property);
/*  461 */     if (deser != null) {
/*  462 */       return deser;
/*      */     }
/*      */     
/*      */ 
/*  466 */     if (!isPotentialBeanType(type.getRawClass())) {
/*  467 */       return null;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*  472 */     return buildBeanDeserializer(config, type, beanDesc, property);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JavaType mapAbstractType(DeserializationConfig config, JavaType type)
/*      */     throws JsonMappingException
/*      */   {
/*      */     for (;;)
/*      */     {
/*  485 */       JavaType next = _mapAbstractType2(config, type);
/*  486 */       if (next == null) {
/*  487 */         return type;
/*      */       }
/*      */       
/*      */ 
/*      */ 
/*      */ 
/*  493 */       Class<?> prevCls = type.getRawClass();
/*  494 */       Class<?> nextCls = next.getRawClass();
/*  495 */       if ((prevCls == nextCls) || (!prevCls.isAssignableFrom(nextCls))) {
/*  496 */         throw new IllegalArgumentException("Invalid abstract type resolution from " + type + " to " + next + ": latter is not a subtype of former");
/*      */       }
/*  498 */       type = next;
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JavaType _mapAbstractType2(DeserializationConfig config, JavaType type)
/*      */     throws JsonMappingException
/*      */   {
/*  509 */     Class<?> currClass = type.getRawClass();
/*  510 */     if (this._factoryConfig.hasAbstractTypeResolvers()) {
/*  511 */       for (AbstractTypeResolver resolver : this._factoryConfig.abstractTypeResolvers()) {
/*  512 */         JavaType concrete = resolver.findTypeMapping(config, type);
/*  513 */         if ((concrete != null) && (concrete.getRawClass() != currClass)) {
/*  514 */           return concrete;
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  520 */     AbstractTypeResolver resolver = config.getAbstractTypeResolver();
/*  521 */     if (resolver != null) {
/*  522 */       JavaType concrete = resolver.findTypeMapping(config, type);
/*  523 */       if ((concrete != null) && (concrete.getRawClass() != currClass)) {
/*  524 */         return concrete;
/*      */       }
/*      */     }
/*  527 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */   protected JavaType materializeAbstractType(DeserializationConfig config, BasicBeanDescription beanDesc)
/*      */     throws JsonMappingException
/*      */   {
/*  536 */     AbstractTypeResolver resolver = config.getAbstractTypeResolver();
/*      */     
/*      */ 
/*  539 */     if ((resolver == null) && (!this._factoryConfig.hasAbstractTypeResolvers())) {
/*  540 */       return null;
/*      */     }
/*      */     
/*  543 */     JavaType abstractType = beanDesc.getType();
/*      */     
/*  545 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/*  546 */     if (intr.findTypeResolver(config, beanDesc.getClassInfo(), abstractType) != null) {
/*  547 */       return null;
/*      */     }
/*      */     
/*  550 */     if (resolver != null) {
/*  551 */       JavaType concrete = resolver.resolveAbstractType(config, abstractType);
/*  552 */       if (concrete != null) {
/*  553 */         return concrete;
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  560 */     for (AbstractTypeResolver r : this._factoryConfig.abstractTypeResolvers()) {
/*  561 */       JavaType concrete = r.resolveAbstractType(config, abstractType);
/*  562 */       if (concrete != null) {
/*  563 */         return concrete;
/*      */       }
/*      */     }
/*  566 */     return null;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonDeserializer<Object> buildBeanDeserializer(DeserializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  589 */     if (type.isAbstract()) {
/*  590 */       return new AbstractDeserializer(type);
/*      */     }
/*      */     
/*  593 */     BeanDeserializerBuilder builder = constructBeanDeserializerBuilder(beanDesc);
/*  594 */     builder.setCreators(findDeserializerCreators(config, beanDesc));
/*      */     
/*  596 */     addBeanProps(config, beanDesc, builder);
/*      */     
/*  598 */     addReferenceProperties(config, beanDesc, builder);
/*      */     
/*      */ 
/*  601 */     if (this._factoryConfig.hasDeserializerModifiers()) {
/*  602 */       for (BeanDeserializerModifier mod : this._factoryConfig.deserializerModifiers()) {
/*  603 */         builder = mod.updateBuilder(config, beanDesc, builder);
/*      */       }
/*      */     }
/*  606 */     JsonDeserializer<?> deserializer = builder.build(property);
/*      */     
/*      */ 
/*  609 */     if (this._factoryConfig.hasDeserializerModifiers()) {
/*  610 */       for (BeanDeserializerModifier mod : this._factoryConfig.deserializerModifiers()) {
/*  611 */         deserializer = mod.modifyDeserializer(config, beanDesc, deserializer);
/*      */       }
/*      */     }
/*  614 */     return deserializer;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   public JsonDeserializer<Object> buildThrowableDeserializer(DeserializationConfig config, JavaType type, BasicBeanDescription beanDesc, BeanProperty property)
/*      */     throws JsonMappingException
/*      */   {
/*  624 */     BeanDeserializerBuilder builder = constructBeanDeserializerBuilder(beanDesc);
/*  625 */     builder.setCreators(findDeserializerCreators(config, beanDesc));
/*      */     
/*  627 */     addBeanProps(config, beanDesc, builder);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  634 */     AnnotatedMethod am = beanDesc.findMethod("initCause", INIT_CAUSE_PARAMS);
/*  635 */     if (am != null) {
/*  636 */       SettableBeanProperty prop = constructSettableProperty(config, beanDesc, "cause", am);
/*  637 */       if (prop != null) {
/*  638 */         builder.addProperty(prop);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  643 */     builder.addIgnorable("localizedMessage");
/*      */     
/*      */ 
/*      */ 
/*  647 */     builder.addIgnorable("message");
/*      */     
/*      */ 
/*  650 */     if (this._factoryConfig.hasDeserializerModifiers()) {
/*  651 */       for (BeanDeserializerModifier mod : this._factoryConfig.deserializerModifiers()) {
/*  652 */         builder = mod.updateBuilder(config, beanDesc, builder);
/*      */       }
/*      */     }
/*  655 */     JsonDeserializer<?> deserializer = builder.build(property);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  660 */     if ((deserializer instanceof BeanDeserializer)) {
/*  661 */       deserializer = new ThrowableDeserializer((BeanDeserializer)deserializer);
/*      */     }
/*      */     
/*      */ 
/*  665 */     if (this._factoryConfig.hasDeserializerModifiers()) {
/*  666 */       for (BeanDeserializerModifier mod : this._factoryConfig.deserializerModifiers()) {
/*  667 */         deserializer = mod.modifyDeserializer(config, beanDesc, deserializer);
/*      */       }
/*      */     }
/*  670 */     return deserializer;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected BeanDeserializerBuilder constructBeanDeserializerBuilder(BasicBeanDescription beanDesc)
/*      */   {
/*  688 */     return new BeanDeserializerBuilder(beanDesc);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected CreatorContainer findDeserializerCreators(DeserializationConfig config, BasicBeanDescription beanDesc)
/*      */     throws JsonMappingException
/*      */   {
/*  701 */     boolean fixAccess = config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS);
/*  702 */     CreatorContainer creators = new CreatorContainer(beanDesc, fixAccess);
/*  703 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/*      */     
/*      */ 
/*      */ 
/*  707 */     if (beanDesc.getType().isConcrete()) {
/*  708 */       java.lang.reflect.Constructor<?> defaultCtor = beanDesc.findDefaultConstructor();
/*  709 */       if (defaultCtor != null) {
/*  710 */         if (fixAccess) {
/*  711 */           org.codehaus.jackson.map.util.ClassUtil.checkAndFixAccess(defaultCtor);
/*      */         }
/*      */         
/*  714 */         creators.setDefaultConstructor(defaultCtor);
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*  719 */     VisibilityChecker<?> vchecker = config.getDefaultVisibilityChecker();
/*  720 */     if (!config.isEnabled(DeserializationConfig.Feature.AUTO_DETECT_CREATORS)) {
/*  721 */       vchecker = vchecker.withCreatorVisibility(org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE);
/*      */     }
/*  723 */     vchecker = config.getAnnotationIntrospector().findAutoDetectVisibility(beanDesc.getClassInfo(), vchecker);
/*      */     
/*  725 */     _addDeserializerConstructors(config, beanDesc, vchecker, intr, creators);
/*  726 */     _addDeserializerFactoryMethods(config, beanDesc, vchecker, intr, creators);
/*  727 */     return creators;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected void _addDeserializerConstructors(DeserializationConfig config, BasicBeanDescription beanDesc, VisibilityChecker<?> vchecker, AnnotationIntrospector intr, CreatorContainer creators)
/*      */     throws JsonMappingException
/*      */   {
/*  735 */     for (org.codehaus.jackson.map.introspect.AnnotatedConstructor ctor : beanDesc.getConstructors()) {
/*  736 */       int argCount = ctor.getParameterCount();
/*  737 */       if (argCount >= 1)
/*      */       {
/*      */ 
/*  740 */         boolean isCreator = intr.hasCreatorAnnotation(ctor);
/*  741 */         boolean isVisible = vchecker.isCreatorVisible(ctor);
/*      */         
/*  743 */         if (argCount == 1)
/*      */         {
/*      */ 
/*      */ 
/*  747 */           org.codehaus.jackson.map.introspect.AnnotatedParameter param = ctor.getParameter(0);
/*  748 */           String name = intr.findPropertyNameForParam(param);
/*  749 */           if ((name == null) || (name.length() == 0)) {
/*  750 */             Class<?> type = ctor.getParameterClass(0);
/*  751 */             if (type == String.class) {
/*  752 */               if ((isCreator) || (isVisible)) {
/*  753 */                 creators.addStringConstructor(ctor);
/*      */               }
/*      */               
/*      */             }
/*  757 */             else if ((type == Integer.TYPE) || (type == Integer.class)) {
/*  758 */               if ((isCreator) || (isVisible)) {
/*  759 */                 creators.addIntConstructor(ctor);
/*      */               }
/*      */               
/*      */             }
/*  763 */             else if ((type == Long.TYPE) || (type == Long.class)) {
/*  764 */               if ((isCreator) || (isVisible)) {
/*  765 */                 creators.addLongConstructor(ctor);
/*      */               }
/*      */               
/*      */ 
/*      */             }
/*  770 */             else if (isCreator) {
/*  771 */               creators.addDelegatingConstructor(ctor);
/*      */             }
/*      */             
/*      */           }
/*      */           else
/*      */           {
/*  777 */             SettableBeanProperty[] properties = new SettableBeanProperty[1];
/*  778 */             properties[0] = constructCreatorProperty(config, beanDesc, name, 0, param);
/*  779 */             creators.addPropertyConstructor(ctor, properties);
/*      */           }
/*  781 */         } else if ((isCreator) || (isVisible))
/*      */         {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  787 */           boolean annotationFound = false;
/*  788 */           boolean notAnnotatedParamFound = false;
/*  789 */           SettableBeanProperty[] properties = new SettableBeanProperty[argCount];
/*  790 */           for (int i = 0; i < argCount; i++) {
/*  791 */             org.codehaus.jackson.map.introspect.AnnotatedParameter param = ctor.getParameter(i);
/*  792 */             String name = param == null ? null : intr.findPropertyNameForParam(param);
/*      */             
/*      */ 
/*  795 */             notAnnotatedParamFound |= ((name == null) || (name.length() == 0));
/*  796 */             annotationFound |= !notAnnotatedParamFound;
/*  797 */             if ((notAnnotatedParamFound) && ((annotationFound) || (isCreator))) {
/*  798 */               throw new IllegalArgumentException("Argument #" + i + " of constructor " + ctor + " has no property name annotation; must have name when multiple-paramater constructor annotated as Creator");
/*      */             }
/*  800 */             properties[i] = constructCreatorProperty(config, beanDesc, name, i, param);
/*      */           }
/*  802 */           if (annotationFound) {
/*  803 */             creators.addPropertyConstructor(ctor, properties);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */   protected void _addDeserializerFactoryMethods(DeserializationConfig config, BasicBeanDescription beanDesc, VisibilityChecker<?> vchecker, AnnotationIntrospector intr, CreatorContainer creators)
/*      */     throws JsonMappingException
/*      */   {
/*  814 */     for (AnnotatedMethod factory : beanDesc.getFactoryMethods()) {
/*  815 */       int argCount = factory.getParameterCount();
/*  816 */       if (argCount >= 1)
/*      */       {
/*      */ 
/*  819 */         boolean isCreator = intr.hasCreatorAnnotation(factory);
/*      */         
/*  821 */         if (argCount == 1)
/*      */         {
/*      */ 
/*      */ 
/*  825 */           String name = intr.findPropertyNameForParam(factory.getParameter(0));
/*  826 */           if ((name == null) || (name.length() == 0)) {
/*  827 */             Class<?> type = factory.getParameterClass(0);
/*  828 */             if (type == String.class) {
/*  829 */               if ((!isCreator) && (!vchecker.isCreatorVisible(factory))) continue;
/*  830 */               creators.addStringFactory(factory); continue;
/*      */             }
/*      */             
/*      */ 
/*  834 */             if ((type == Integer.TYPE) || (type == Integer.class)) {
/*  835 */               if ((!isCreator) && (!vchecker.isCreatorVisible(factory))) continue;
/*  836 */               creators.addIntFactory(factory); continue;
/*      */             }
/*      */             
/*      */ 
/*  840 */             if ((type == Long.TYPE) || (type == Long.class)) {
/*  841 */               if ((!isCreator) && (!vchecker.isCreatorVisible(factory))) continue;
/*  842 */               creators.addLongFactory(factory); continue;
/*      */             }
/*      */             
/*      */ 
/*  846 */             if (!intr.hasCreatorAnnotation(factory)) continue;
/*  847 */             creators.addDelegatingFactory(factory); continue;
/*      */           }
/*      */           
/*      */ 
/*      */         }
/*      */         else
/*      */         {
/*      */ 
/*  855 */           if (!intr.hasCreatorAnnotation(factory)) {
/*      */             continue;
/*      */           }
/*      */         }
/*      */         
/*  860 */         SettableBeanProperty[] properties = new SettableBeanProperty[argCount];
/*  861 */         for (int i = 0; i < argCount; i++) {
/*  862 */           org.codehaus.jackson.map.introspect.AnnotatedParameter param = factory.getParameter(i);
/*  863 */           String name = intr.findPropertyNameForParam(param);
/*      */           
/*  865 */           if ((name == null) || (name.length() == 0)) {
/*  866 */             throw new IllegalArgumentException("Argument #" + i + " of factory method " + factory + " has no property name annotation; must have when multiple-paramater static method annotated as Creator");
/*      */           }
/*  868 */           properties[i] = constructCreatorProperty(config, beanDesc, name, i, param);
/*      */         }
/*  870 */         creators.addPropertyFactory(factory, properties);
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void addBeanProps(DeserializationConfig config, BasicBeanDescription beanDesc, BeanDeserializerBuilder builder)
/*      */     throws JsonMappingException
/*      */   {
/*  886 */     VisibilityChecker<?> vchecker = config.getDefaultVisibilityChecker();
/*      */     
/*  888 */     if (!config.isEnabled(DeserializationConfig.Feature.AUTO_DETECT_SETTERS)) {
/*  889 */       vchecker = vchecker.withSetterVisibility(org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE);
/*      */     }
/*  891 */     if (!config.isEnabled(DeserializationConfig.Feature.AUTO_DETECT_FIELDS)) {
/*  892 */       vchecker = vchecker.withFieldVisibility(org.codehaus.jackson.annotate.JsonAutoDetect.Visibility.NONE);
/*      */     }
/*      */     
/*  895 */     vchecker = config.getAnnotationIntrospector().findAutoDetectVisibility(beanDesc.getClassInfo(), vchecker);
/*      */     
/*  897 */     Map<String, AnnotatedMethod> setters = beanDesc.findSetters(vchecker);
/*      */     
/*  899 */     AnnotatedMethod anySetter = beanDesc.findAnySetter();
/*      */     
/*      */ 
/*  902 */     AnnotationIntrospector intr = config.getAnnotationIntrospector();
/*  903 */     boolean ignoreAny = false;
/*      */     
/*  905 */     Boolean B = intr.findIgnoreUnknownProperties(beanDesc.getClassInfo());
/*  906 */     if (B != null) {
/*  907 */       ignoreAny = B.booleanValue();
/*  908 */       builder.setIgnoreUnknownProperties(ignoreAny);
/*      */     }
/*      */     
/*      */ 
/*  912 */     HashSet<String> ignored = ArrayBuilders.arrayToSet(intr.findPropertiesToIgnore(beanDesc.getClassInfo()));
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  919 */     for (String propName : ignored) {
/*  920 */       builder.addIgnorable(propName);
/*      */     }
/*      */     
/*  923 */     org.codehaus.jackson.map.introspect.AnnotatedClass ac = beanDesc.getClassInfo();
/*  924 */     for (AnnotatedMethod am : ac.ignoredMemberMethods()) {
/*  925 */       String name = beanDesc.okNameForSetter(am);
/*  926 */       if (name != null) {
/*  927 */         builder.addIgnorable(name);
/*      */       }
/*      */     }
/*  930 */     for (AnnotatedField af : ac.ignoredFields()) {
/*  931 */       builder.addIgnorable(af.getName());
/*      */     }
/*      */     
/*      */ 
/*  935 */     java.util.HashMap<Class<?>, Boolean> ignoredTypes = new java.util.HashMap();
/*      */     
/*      */ 
/*  938 */     for (Map.Entry<String, AnnotatedMethod> en : setters.entrySet()) {
/*  939 */       String name = (String)en.getKey();
/*  940 */       if (!ignored.contains(name)) {
/*  941 */         AnnotatedMethod setter = (AnnotatedMethod)en.getValue();
/*      */         
/*  943 */         Class<?> type = setter.getParameterClass(0);
/*  944 */         if (isIgnorableType(config, beanDesc, type, ignoredTypes))
/*      */         {
/*  946 */           builder.addIgnorable(name);
/*      */         }
/*      */         else {
/*  949 */           SettableBeanProperty prop = constructSettableProperty(config, beanDesc, name, setter);
/*  950 */           if (prop != null)
/*  951 */             builder.addProperty(prop);
/*      */         }
/*      */       }
/*      */     }
/*  955 */     if (anySetter != null) {
/*  956 */       builder.setAnySetter(constructAnySetter(config, beanDesc, anySetter));
/*      */     }
/*      */     
/*  959 */     HashSet<String> addedProps = new HashSet(setters.keySet());
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*  964 */     java.util.LinkedHashMap<String, AnnotatedField> fieldsByProp = beanDesc.findDeserializableFields(vchecker, addedProps);
/*  965 */     for (Map.Entry<String, AnnotatedField> en : fieldsByProp.entrySet()) {
/*  966 */       String name = (String)en.getKey();
/*  967 */       if ((!ignored.contains(name)) && (!builder.hasProperty(name))) {
/*  968 */         AnnotatedField field = (AnnotatedField)en.getValue();
/*      */         
/*  970 */         Class<?> type = field.getRawType();
/*  971 */         if (isIgnorableType(config, beanDesc, type, ignoredTypes))
/*      */         {
/*  973 */           builder.addIgnorable(name);
/*      */         }
/*      */         else {
/*  976 */           SettableBeanProperty prop = constructSettableProperty(config, beanDesc, name, field);
/*  977 */           if (prop != null) {
/*  978 */             builder.addProperty(prop);
/*  979 */             addedProps.add(name);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  990 */     if (config.isEnabled(DeserializationConfig.Feature.USE_GETTERS_AS_SETTERS))
/*      */     {
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*  996 */       Map<String, AnnotatedMethod> getters = beanDesc.findGetters(vchecker, addedProps);
/*      */       
/*  998 */       for (Map.Entry<String, AnnotatedMethod> en : getters.entrySet()) {
/*  999 */         AnnotatedMethod getter = (AnnotatedMethod)en.getValue();
/*      */         
/* 1001 */         Class<?> rt = getter.getRawType();
/* 1002 */         if ((java.util.Collection.class.isAssignableFrom(rt)) || (Map.class.isAssignableFrom(rt)))
/*      */         {
/*      */ 
/* 1005 */           String name = (String)en.getKey();
/* 1006 */           if ((!ignored.contains(name)) && (!builder.hasProperty(name))) {
/* 1007 */             builder.addProperty(constructSetterlessProperty(config, beanDesc, name, getter));
/* 1008 */             addedProps.add(name);
/*      */           }
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected void addReferenceProperties(DeserializationConfig config, BasicBeanDescription beanDesc, BeanDeserializerBuilder builder)
/*      */     throws JsonMappingException
/*      */   {
/* 1026 */     Map<String, org.codehaus.jackson.map.introspect.AnnotatedMember> refs = beanDesc.findBackReferenceProperties();
/* 1027 */     if (refs != null) {
/* 1028 */       for (Map.Entry<String, org.codehaus.jackson.map.introspect.AnnotatedMember> en : refs.entrySet()) {
/* 1029 */         String name = (String)en.getKey();
/* 1030 */         org.codehaus.jackson.map.introspect.AnnotatedMember m = (org.codehaus.jackson.map.introspect.AnnotatedMember)en.getValue();
/* 1031 */         if ((m instanceof AnnotatedMethod)) {
/* 1032 */           builder.addBackReferenceProperty(name, constructSettableProperty(config, beanDesc, m.getName(), (AnnotatedMethod)m));
/*      */         }
/*      */         else {
/* 1035 */           builder.addBackReferenceProperty(name, constructSettableProperty(config, beanDesc, m.getName(), (AnnotatedField)m));
/*      */         }
/*      */       }
/*      */     }
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SettableAnyProperty constructAnySetter(DeserializationConfig config, BasicBeanDescription beanDesc, AnnotatedMethod setter)
/*      */     throws JsonMappingException
/*      */   {
/* 1051 */     if (config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 1052 */       setter.fixAccess();
/*      */     }
/*      */     
/* 1055 */     JavaType type = beanDesc.bindingsForBeanType().resolveType(setter.getParameterType(1));
/* 1056 */     BeanProperty.Std property = new BeanProperty.Std(setter.getName(), type, beanDesc.getClassAnnotations(), setter);
/* 1057 */     type = resolveType(config, beanDesc, type, setter, property);
/*      */     
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/* 1064 */     JsonDeserializer<Object> deser = findDeserializerFromAnnotation(config, setter, property);
/* 1065 */     if (deser != null) {
/* 1066 */       SettableAnyProperty prop = new SettableAnyProperty(property, setter, type);
/* 1067 */       prop.setValueDeserializer(deser);
/* 1068 */       return prop;
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1073 */     type = modifyTypeByAnnotation(config, setter, type, property.getName());
/* 1074 */     return new SettableAnyProperty(property, setter, type);
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SettableBeanProperty constructSettableProperty(DeserializationConfig config, BasicBeanDescription beanDesc, String name, AnnotatedMethod setter)
/*      */     throws JsonMappingException
/*      */   {
/* 1093 */     if (config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 1094 */       setter.fixAccess();
/*      */     }
/*      */     
/*      */ 
/* 1098 */     JavaType t0 = beanDesc.bindingsForBeanType().resolveType(setter.getParameterType(0));
/* 1099 */     BeanProperty.Std property = new BeanProperty.Std(name, t0, beanDesc.getClassAnnotations(), setter);
/* 1100 */     JavaType type = resolveType(config, beanDesc, t0, setter, property);
/*      */     
/* 1102 */     if (type != t0) {
/* 1103 */       property = property.withType(type);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/*      */ 
/* 1109 */     JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(config, setter, property);
/* 1110 */     type = modifyTypeByAnnotation(config, setter, type, name);
/* 1111 */     TypeDeserializer typeDeser = (TypeDeserializer)type.getTypeHandler();
/* 1112 */     SettableBeanProperty prop = new SettableBeanProperty.MethodProperty(name, type, typeDeser, beanDesc.getClassAnnotations(), setter);
/*      */     
/* 1114 */     if (propDeser != null) {
/* 1115 */       prop.setValueDeserializer(propDeser);
/*      */     }
/*      */     
/* 1118 */     AnnotationIntrospector.ReferenceProperty ref = config.getAnnotationIntrospector().findReferenceType(setter);
/* 1119 */     if ((ref != null) && (ref.isManagedReference())) {
/* 1120 */       prop.setManagedReferenceName(ref.getName());
/*      */     }
/* 1122 */     return prop;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */   protected SettableBeanProperty constructSettableProperty(DeserializationConfig config, BasicBeanDescription beanDesc, String name, AnnotatedField field)
/*      */     throws JsonMappingException
/*      */   {
/* 1130 */     if (config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 1131 */       field.fixAccess();
/*      */     }
/* 1133 */     JavaType t0 = beanDesc.bindingsForBeanType().resolveType(field.getGenericType());
/* 1134 */     BeanProperty.Std property = new BeanProperty.Std(name, t0, beanDesc.getClassAnnotations(), field);
/* 1135 */     JavaType type = resolveType(config, beanDesc, t0, field, property);
/*      */     
/* 1137 */     if (type != t0) {
/* 1138 */       property = property.withType(type);
/*      */     }
/*      */     
/*      */ 
/*      */ 
/* 1143 */     JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(config, field, property);
/* 1144 */     type = modifyTypeByAnnotation(config, field, type, name);
/* 1145 */     TypeDeserializer typeDeser = (TypeDeserializer)type.getTypeHandler();
/* 1146 */     SettableBeanProperty prop = new SettableBeanProperty.FieldProperty(name, type, typeDeser, beanDesc.getClassAnnotations(), field);
/*      */     
/* 1148 */     if (propDeser != null) {
/* 1149 */       prop.setValueDeserializer(propDeser);
/*      */     }
/*      */     
/* 1152 */     AnnotationIntrospector.ReferenceProperty ref = config.getAnnotationIntrospector().findReferenceType(field);
/* 1153 */     if ((ref != null) && (ref.isManagedReference())) {
/* 1154 */       prop.setManagedReferenceName(ref.getName());
/*      */     }
/* 1156 */     return prop;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected SettableBeanProperty constructSetterlessProperty(DeserializationConfig config, BasicBeanDescription beanDesc, String name, AnnotatedMethod getter)
/*      */     throws JsonMappingException
/*      */   {
/* 1171 */     if (config.isEnabled(DeserializationConfig.Feature.CAN_OVERRIDE_ACCESS_MODIFIERS)) {
/* 1172 */       getter.fixAccess();
/*      */     }
/*      */     
/* 1175 */     JavaType type = getter.getType(beanDesc.bindingsForBeanType());
/*      */     
/*      */ 
/*      */ 
/* 1179 */     BeanProperty.Std property = new BeanProperty.Std(name, type, beanDesc.getClassAnnotations(), getter);
/*      */     
/* 1181 */     JsonDeserializer<Object> propDeser = findDeserializerFromAnnotation(config, getter, property);
/* 1182 */     type = modifyTypeByAnnotation(config, getter, type, name);
/* 1183 */     TypeDeserializer typeDeser = (TypeDeserializer)type.getTypeHandler();
/* 1184 */     SettableBeanProperty prop = new SettableBeanProperty.SetterlessProperty(name, type, typeDeser, beanDesc.getClassAnnotations(), getter);
/*      */     
/* 1186 */     if (propDeser != null) {
/* 1187 */       prop.setValueDeserializer(propDeser);
/*      */     }
/* 1189 */     return prop;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected boolean isPotentialBeanType(Class<?> type)
/*      */   {
/* 1208 */     String typeStr = org.codehaus.jackson.map.util.ClassUtil.canBeABeanType(type);
/* 1209 */     if (typeStr != null) {
/* 1210 */       throw new IllegalArgumentException("Can not deserialize Class " + type.getName() + " (of type " + typeStr + ") as a Bean");
/*      */     }
/* 1212 */     if (org.codehaus.jackson.map.util.ClassUtil.isProxyType(type)) {
/* 1213 */       throw new IllegalArgumentException("Can not deserialize Proxy class " + type.getName() + " as a Bean");
/*      */     }
/*      */     
/* 1216 */     typeStr = org.codehaus.jackson.map.util.ClassUtil.isLocalType(type);
/* 1217 */     if (typeStr != null) {
/* 1218 */       throw new IllegalArgumentException("Can not deserialize Class " + type.getName() + " (of type " + typeStr + ") as a Bean");
/*      */     }
/* 1220 */     return true;
/*      */   }
/*      */   
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */ 
/*      */   protected boolean isIgnorableType(DeserializationConfig config, BasicBeanDescription beanDesc, Class<?> type, Map<Class<?>, Boolean> ignoredTypes)
/*      */   {
/* 1230 */     Boolean status = (Boolean)ignoredTypes.get(type);
/* 1231 */     if (status == null) {
/* 1232 */       BasicBeanDescription desc = (BasicBeanDescription)config.introspectClassAnnotations(type);
/* 1233 */       status = config.getAnnotationIntrospector().isIgnorableType(desc.getClassInfo());
/*      */       
/* 1235 */       if (status == null) {
/* 1236 */         status = Boolean.FALSE;
/*      */       }
/*      */     }
/* 1239 */     return status.booleanValue();
/*      */   }
/*      */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\codehaus\jackson\map\deser\BeanDeserializerFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */