/*     */ package org.hibernate.annotations.common.reflection.java;
/*     */ 
/*     */ import java.lang.reflect.Field;
/*     */ import java.lang.reflect.Method;
/*     */ import java.lang.reflect.Modifier;
/*     */ import java.util.LinkedList;
/*     */ import java.util.List;
/*     */ import org.hibernate.annotations.common.reflection.Filter;
/*     */ import org.hibernate.annotations.common.reflection.ReflectionUtil;
/*     */ import org.hibernate.annotations.common.reflection.XClass;
/*     */ import org.hibernate.annotations.common.reflection.XMethod;
/*     */ import org.hibernate.annotations.common.reflection.XProperty;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.CompoundTypeEnvironment;
/*     */ import org.hibernate.annotations.common.reflection.java.generics.TypeEnvironment;
/*     */ 
/*     */ 
/*     */ 
/*     */ class JavaXClass
/*     */   extends JavaXAnnotatedElement
/*     */   implements XClass
/*     */ {
/*     */   private final TypeEnvironment context;
/*     */   private final Class clazz;
/*     */   
/*     */   public JavaXClass(Class clazz, TypeEnvironment env, JavaReflectionManager factory)
/*     */   {
/*  27 */     super(clazz, factory);
/*  28 */     this.clazz = clazz;
/*  29 */     this.context = env;
/*     */   }
/*     */   
/*     */   public String getName() {
/*  33 */     return toClass().getName();
/*     */   }
/*     */   
/*     */   public XClass getSuperclass() {
/*  37 */     return getFactory().toXClass(toClass().getSuperclass(), CompoundTypeEnvironment.create(getTypeEnvironment(), getFactory().getTypeEnvironment(toClass())));
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public XClass[] getInterfaces()
/*     */   {
/*  46 */     Class[] classes = toClass().getInterfaces();
/*  47 */     int length = classes.length;
/*  48 */     XClass[] xClasses = new XClass[length];
/*  49 */     if (length != 0) {
/*  50 */       TypeEnvironment environment = CompoundTypeEnvironment.create(getTypeEnvironment(), getFactory().getTypeEnvironment(toClass()));
/*     */       
/*     */ 
/*     */ 
/*  54 */       for (int index = 0; index < length; index++) {
/*  55 */         xClasses[index] = getFactory().toXClass(classes[index], environment);
/*     */       }
/*     */     }
/*  58 */     return xClasses;
/*     */   }
/*     */   
/*     */   public boolean isInterface() {
/*  62 */     return toClass().isInterface();
/*     */   }
/*     */   
/*     */   public boolean isAbstract() {
/*  66 */     return Modifier.isAbstract(toClass().getModifiers());
/*     */   }
/*     */   
/*     */   public boolean isPrimitive() {
/*  70 */     return toClass().isPrimitive();
/*     */   }
/*     */   
/*     */   public boolean isEnum() {
/*  74 */     return toClass().isEnum();
/*     */   }
/*     */   
/*     */   private List<XProperty> getDeclaredFieldProperties(Filter filter) {
/*  78 */     List<XProperty> result = new LinkedList();
/*  79 */     for (Field f : toClass().getDeclaredFields()) {
/*  80 */       if (ReflectionUtil.isProperty(f, getTypeEnvironment().bind(f.getGenericType()), filter)) {
/*  81 */         result.add(getFactory().getXProperty(f, getTypeEnvironment()));
/*     */       }
/*     */     }
/*  84 */     return result;
/*     */   }
/*     */   
/*     */   private List<XProperty> getDeclaredMethodProperties(Filter filter) {
/*  88 */     List<XProperty> result = new LinkedList();
/*  89 */     for (Method m : toClass().getDeclaredMethods()) {
/*  90 */       if (ReflectionUtil.isProperty(m, getTypeEnvironment().bind(m.getGenericReturnType()), filter)) {
/*  91 */         result.add(getFactory().getXProperty(m, getTypeEnvironment()));
/*     */       }
/*     */     }
/*  94 */     return result;
/*     */   }
/*     */   
/*     */   public List<XProperty> getDeclaredProperties(String accessType) {
/*  98 */     return getDeclaredProperties(accessType, XClass.DEFAULT_FILTER);
/*     */   }
/*     */   
/*     */   public List<XProperty> getDeclaredProperties(String accessType, Filter filter) {
/* 102 */     if (accessType.equals("field")) {
/* 103 */       return getDeclaredFieldProperties(filter);
/*     */     }
/* 105 */     if (accessType.equals("property")) {
/* 106 */       return getDeclaredMethodProperties(filter);
/*     */     }
/* 108 */     throw new IllegalArgumentException("Unknown access type " + accessType);
/*     */   }
/*     */   
/*     */   public List<XMethod> getDeclaredMethods() {
/* 112 */     List<XMethod> result = new LinkedList();
/* 113 */     for (Method m : toClass().getDeclaredMethods()) {
/* 114 */       result.add(getFactory().getXMethod(m, getTypeEnvironment()));
/*     */     }
/* 116 */     return result;
/*     */   }
/*     */   
/*     */   public Class<?> toClass() {
/* 120 */     return this.clazz;
/*     */   }
/*     */   
/*     */   public boolean isAssignableFrom(XClass c) {
/* 124 */     return toClass().isAssignableFrom(((JavaXClass)c).toClass());
/*     */   }
/*     */   
/*     */   boolean isArray() {
/* 128 */     return toClass().isArray();
/*     */   }
/*     */   
/*     */   TypeEnvironment getTypeEnvironment() {
/* 132 */     return this.context;
/*     */   }
/*     */   
/*     */   public String toString()
/*     */   {
/* 137 */     return getName();
/*     */   }
/*     */ }


/* Location:              /Users/anandkum/Downloads/hibernate-commons-annotations-3.0.0.ga.jar!/org/hibernate/annotations/common/reflection/java/JavaXClass.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */