/*     */ package org.xerial.snappy;
/*     */ 
/*     */ import java.io.ByteArrayInputStream;
/*     */ import java.io.IOException;
/*     */ import java.io.InputStream;
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ public class SnappyInputStream
/*     */   extends InputStream
/*     */ {
/*  40 */   private boolean finishedReading = false;
/*     */   
/*     */   protected final InputStream in;
/*     */   private byte[] compressed;
/*     */   private byte[] uncompressed;
/*  45 */   private int uncompressedCursor = 0;
/*  46 */   private int uncompressedLimit = 0;
/*     */   
/*  48 */   private byte[] chunkSizeBuf = new byte[4];
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public SnappyInputStream(InputStream input)
/*     */     throws IOException
/*     */   {
/*  57 */     this.in = input;
/*  58 */     readHeader();
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public void close()
/*     */     throws IOException
/*     */   {
/*  69 */     this.compressed = null;
/*  70 */     this.uncompressed = null;
/*  71 */     if (this.in != null)
/*  72 */       this.in.close();
/*     */   }
/*     */   
/*     */   protected void readHeader() throws IOException {
/*  76 */     byte[] header = new byte[SnappyCodec.headerSize()];
/*  77 */     int readBytes = 0;
/*  78 */     while (readBytes < header.length) {
/*  79 */       int ret = this.in.read(header, readBytes, header.length - readBytes);
/*  80 */       if (ret == -1)
/*     */         break;
/*  82 */       readBytes += ret;
/*     */     }
/*     */     
/*     */ 
/*  86 */     if ((readBytes < header.length) || (header[0] != SnappyCodec.MAGIC_HEADER[0]))
/*     */     {
/*  88 */       readFully(header, readBytes);
/*  89 */       return;
/*     */     }
/*     */     
/*  92 */     SnappyCodec codec = SnappyCodec.readHeader(new ByteArrayInputStream(header));
/*  93 */     if (codec.isValidMagicHeader())
/*     */     {
/*  95 */       if (codec.version < 1) {
/*  96 */         throw new IOException(String.format("compressed with imcompatible codec version %d. At least version %d is required", new Object[] { Integer.valueOf(codec.version), Integer.valueOf(1) }));
/*     */       }
/*     */       
/*     */ 
/*     */     }
/*     */     else
/*     */     {
/* 103 */       readFully(header, readBytes);
/* 104 */       return;
/*     */     }
/*     */   }
/*     */   
/*     */   protected void readFully(byte[] fragment, int fragmentLength) throws IOException
/*     */   {
/* 110 */     this.compressed = new byte[Math.max(8192, fragmentLength)];
/* 111 */     System.arraycopy(fragment, 0, this.compressed, 0, fragmentLength);
/* 112 */     int cursor = fragmentLength;
/* 113 */     for (int readBytes = 0; (readBytes = this.in.read(this.compressed, cursor, this.compressed.length - cursor)) != -1;) {
/* 114 */       cursor += readBytes;
/* 115 */       if (cursor >= this.compressed.length) {
/* 116 */         byte[] newBuf = new byte[this.compressed.length * 2];
/* 117 */         System.arraycopy(this.compressed, 0, newBuf, 0, this.compressed.length);
/* 118 */         this.compressed = newBuf;
/*     */       }
/*     */     }
/*     */     
/* 122 */     this.finishedReading = true;
/*     */     
/*     */ 
/* 125 */     int uncompressedLength = Snappy.uncompressedLength(this.compressed, 0, cursor);
/* 126 */     this.uncompressed = new byte[uncompressedLength];
/* 127 */     Snappy.uncompress(this.compressed, 0, cursor, this.uncompressed, 0);
/* 128 */     this.uncompressedCursor = 0;
/* 129 */     this.uncompressedLimit = uncompressedLength;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(byte[] b, int off, int len)
/*     */     throws IOException
/*     */   {
/* 142 */     return rawRead(b, off, len);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int rawRead(Object array, int byteOffset, int byteLength)
/*     */     throws IOException
/*     */   {
/* 155 */     int writtenBytes = 0;
/* 156 */     while (writtenBytes < byteLength) {
/* 157 */       if (this.uncompressedCursor >= this.uncompressedLimit) {
/* 158 */         if (!hasNextChunk())
/*     */         {
/*     */ 
/* 161 */           return writtenBytes == 0 ? -1 : writtenBytes;
/*     */         }
/*     */       } else {
/* 164 */         int bytesToWrite = Math.min(this.uncompressedLimit - this.uncompressedCursor, byteLength - writtenBytes);
/* 165 */         Snappy.arrayCopy(this.uncompressed, this.uncompressedCursor, bytesToWrite, array, byteOffset + writtenBytes);
/* 166 */         writtenBytes += bytesToWrite;
/* 167 */         this.uncompressedCursor += bytesToWrite;
/*     */       }
/*     */     }
/* 170 */     return writtenBytes;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(long[] d, int off, int len)
/*     */     throws IOException
/*     */   {
/* 187 */     return rawRead(d, off * 8, len * 8);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(long[] d)
/*     */     throws IOException
/*     */   {
/* 199 */     return read(d, 0, d.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(double[] d, int off, int len)
/*     */     throws IOException
/*     */   {
/* 216 */     return rawRead(d, off * 8, len * 8);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(double[] d)
/*     */     throws IOException
/*     */   {
/* 228 */     return read(d, 0, d.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(int[] d)
/*     */     throws IOException
/*     */   {
/* 240 */     return read(d, 0, d.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(int[] d, int off, int len)
/*     */     throws IOException
/*     */   {
/* 257 */     return rawRead(d, off * 4, len * 4);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(float[] d, int off, int len)
/*     */     throws IOException
/*     */   {
/* 274 */     return rawRead(d, off * 4, len * 4);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(float[] d)
/*     */     throws IOException
/*     */   {
/* 286 */     return read(d, 0, d.length);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(short[] d, int off, int len)
/*     */     throws IOException
/*     */   {
/* 303 */     return rawRead(d, off * 2, len * 2);
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read(short[] d)
/*     */     throws IOException
/*     */   {
/* 315 */     return read(d, 0, d.length);
/*     */   }
/*     */   
/*     */   protected boolean hasNextChunk() throws IOException {
/* 319 */     if (this.finishedReading) {
/* 320 */       return false;
/*     */     }
/* 322 */     this.uncompressedCursor = 0;
/* 323 */     this.uncompressedLimit = 0;
/*     */     
/* 325 */     int readBytes = 0;
/* 326 */     while (readBytes < 4) {
/* 327 */       int ret = this.in.read(this.chunkSizeBuf, readBytes, 4 - readBytes);
/* 328 */       if (ret == -1) {
/* 329 */         this.finishedReading = true;
/* 330 */         return false;
/*     */       }
/* 332 */       readBytes += ret;
/*     */     }
/* 334 */     int chunkSize = SnappyOutputStream.readInt(this.chunkSizeBuf, 0);
/*     */     
/* 336 */     if ((this.compressed == null) || (chunkSize > this.compressed.length)) {
/* 337 */       this.compressed = new byte[chunkSize];
/*     */     }
/* 339 */     readBytes = 0;
/* 340 */     while (readBytes < chunkSize) {
/* 341 */       int ret = this.in.read(this.compressed, readBytes, chunkSize - readBytes);
/* 342 */       if (ret == -1)
/*     */         break;
/* 344 */       readBytes += ret;
/*     */     }
/* 346 */     if (readBytes < chunkSize) {
/* 347 */       throw new IOException("failed to read chunk");
/*     */     }
/*     */     try {
/* 350 */       int uncompressedLength = Snappy.uncompressedLength(this.compressed, 0, chunkSize);
/* 351 */       if ((this.uncompressed == null) || (uncompressedLength > this.uncompressed.length)) {
/* 352 */         this.uncompressed = new byte[uncompressedLength];
/*     */       }
/* 354 */       int actualUncompressedLength = Snappy.uncompress(this.compressed, 0, chunkSize, this.uncompressed, 0);
/* 355 */       if (uncompressedLength != actualUncompressedLength) {
/* 356 */         throw new IOException("invalid uncompressed byte size");
/*     */       }
/* 358 */       this.uncompressedLimit = actualUncompressedLength;
/*     */     }
/*     */     catch (IOException e) {
/* 361 */       throw new IOException("failed to uncompress the chunk: " + e.getMessage());
/*     */     }
/*     */     
/* 364 */     return true;
/*     */   }
/*     */   
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */ 
/*     */   public int read()
/*     */     throws IOException
/*     */   {
/* 379 */     if (this.uncompressedCursor < this.uncompressedLimit) {
/* 380 */       return this.uncompressed[(this.uncompressedCursor++)] & 0xFF;
/*     */     }
/*     */     
/* 383 */     if (hasNextChunk()) {
/* 384 */       return read();
/*     */     }
/* 386 */     return -1;
/*     */   }
/*     */ }


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\org\xerial\snappy\SnappyInputStream.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */