package prefuse.action.layout.graph;

import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.io.PrintStream;
import java.util.Iterator;
import prefuse.Visualization;
import prefuse.action.layout.Layout;
import prefuse.data.Graph;
import prefuse.data.Schema;
import prefuse.data.tuple.TupleSet;
import prefuse.util.PrefuseLib;
import prefuse.util.force.DragForce;
import prefuse.util.force.ForceItem;
import prefuse.util.force.ForceSimulator;
import prefuse.util.force.NBodyForce;
import prefuse.util.force.SpringForce;
import prefuse.visual.EdgeItem;
import prefuse.visual.NodeItem;
import prefuse.visual.VisualItem;

public class ForceDirectedLayout
  extends Layout
{
  private ForceSimulator m_fsim;
  private long m_lasttime = -1L;
  private long m_maxstep = 50L;
  private boolean m_runonce;
  private int m_iterations = 100;
  private boolean m_enforceBounds;
  protected transient VisualItem referrer;
  protected String m_nodeGroup;
  protected String m_edgeGroup;
  public static final String FORCEITEM = "_forceItem";
  public static final Schema FORCEITEM_SCHEMA = new Schema();
  
  public ForceDirectedLayout(String paramString)
  {
    this(paramString, false, false);
  }
  
  public ForceDirectedLayout(String paramString, boolean paramBoolean)
  {
    this(paramString, paramBoolean, false);
  }
  
  public ForceDirectedLayout(String paramString, boolean paramBoolean1, boolean paramBoolean2)
  {
    super(paramString);
    this.m_nodeGroup = PrefuseLib.getGroupName(paramString, Graph.NODES);
    this.m_edgeGroup = PrefuseLib.getGroupName(paramString, Graph.EDGES);
    this.m_enforceBounds = paramBoolean1;
    this.m_runonce = paramBoolean2;
    this.m_fsim = new ForceSimulator();
    this.m_fsim.addForce(new NBodyForce());
    this.m_fsim.addForce(new SpringForce());
    this.m_fsim.addForce(new DragForce());
  }
  
  public ForceDirectedLayout(String paramString, ForceSimulator paramForceSimulator, boolean paramBoolean)
  {
    this(paramString, paramForceSimulator, paramBoolean, false);
  }
  
  public ForceDirectedLayout(String paramString, ForceSimulator paramForceSimulator, boolean paramBoolean1, boolean paramBoolean2)
  {
    super(paramString);
    this.m_nodeGroup = PrefuseLib.getGroupName(paramString, Graph.NODES);
    this.m_edgeGroup = PrefuseLib.getGroupName(paramString, Graph.EDGES);
    this.m_enforceBounds = paramBoolean1;
    this.m_runonce = paramBoolean2;
    this.m_fsim = paramForceSimulator;
  }
  
  public long getMaxTimeStep()
  {
    return this.m_maxstep;
  }
  
  public void setMaxTimeStep(long paramLong)
  {
    this.m_maxstep = paramLong;
  }
  
  public ForceSimulator getForceSimulator()
  {
    return this.m_fsim;
  }
  
  public void setForceSimulator(ForceSimulator paramForceSimulator)
  {
    this.m_fsim = paramForceSimulator;
  }
  
  public int getIterations()
  {
    return this.m_iterations;
  }
  
  public void setIterations(int paramInt)
  {
    if (paramInt < 1) {
      throw new IllegalArgumentException("Iterations must be a positive number!");
    }
    this.m_iterations = paramInt;
  }
  
  public void run(double paramDouble)
  {
    long l2;
    if (this.m_runonce)
    {
      Point2D localPoint2D = getLayoutAnchor();
      Iterator localIterator = this.m_vis.visibleItems(this.m_nodeGroup);
      while (localIterator.hasNext())
      {
        NodeItem localNodeItem = (NodeItem)localIterator.next();
        localNodeItem.setX(localPoint2D.getX());
        localNodeItem.setY(localPoint2D.getY());
      }
      this.m_fsim.clear();
      l2 = 1000L;
      initSimulator(this.m_fsim);
      for (int i = 0; i < this.m_iterations; i++)
      {
        l2 = (l2 * (1.0D - i / this.m_iterations));
        long l3 = l2 + 50L;
        this.m_fsim.runSimulator(l3);
        if (i % 10 == 0) {
          System.out.println("iter: " + i);
        }
      }
      updateNodePositions();
    }
    else
    {
      if (this.m_lasttime == -1L) {
        this.m_lasttime = (System.currentTimeMillis() - 20L);
      }
      long l1 = System.currentTimeMillis();
      l2 = Math.min(this.m_maxstep, l1 - this.m_lasttime);
      this.m_lasttime = l1;
      this.m_fsim.clear();
      initSimulator(this.m_fsim);
      this.m_fsim.runSimulator(l2);
      updateNodePositions();
    }
    if (paramDouble == 1.0D) {
      reset();
    }
  }
  
  private void updateNodePositions()
  {
    Rectangle2D localRectangle2D1 = getLayoutBounds();
    double d1 = 0.0D;
    double d2 = 0.0D;
    double d3 = 0.0D;
    double d4 = 0.0D;
    if (localRectangle2D1 != null)
    {
      d1 = localRectangle2D1.getMinX();
      d3 = localRectangle2D1.getMinY();
      d2 = localRectangle2D1.getMaxX();
      d4 = localRectangle2D1.getMaxY();
    }
    Iterator localIterator = this.m_vis.visibleItems(this.m_nodeGroup);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      ForceItem localForceItem = (ForceItem)localVisualItem.get("_forceItem");
      if (localVisualItem.isFixed())
      {
        localForceItem.force[0] = 0.0F;
        localForceItem.force[1] = 0.0F;
        localForceItem.velocity[0] = 0.0F;
        localForceItem.velocity[1] = 0.0F;
        if (Double.isNaN(localVisualItem.getX()))
        {
          super.setX(localVisualItem, this.referrer, 0.0D);
          super.setY(localVisualItem, this.referrer, 0.0D);
        }
      }
      else
      {
        double d5 = localForceItem.location[0];
        double d6 = localForceItem.location[1];
        if ((this.m_enforceBounds) && (localRectangle2D1 != null))
        {
          Rectangle2D localRectangle2D2 = localVisualItem.getBounds();
          double d7 = localRectangle2D2.getWidth() / 2.0D;
          double d8 = localRectangle2D2.getHeight() / 2.0D;
          if (d5 + d7 > d2) {
            d5 = d2 - d7;
          }
          if (d5 - d7 < d1) {
            d5 = d1 + d7;
          }
          if (d6 + d8 > d4) {
            d6 = d4 - d8;
          }
          if (d6 - d8 < d3) {
            d6 = d3 + d8;
          }
        }
        super.setX(localVisualItem, this.referrer, d5);
        super.setY(localVisualItem, this.referrer, d6);
      }
    }
  }
  
  public void reset()
  {
    Iterator localIterator = this.m_vis.visibleItems(this.m_nodeGroup);
    while (localIterator.hasNext())
    {
      VisualItem localVisualItem = (VisualItem)localIterator.next();
      ForceItem localForceItem = (ForceItem)localVisualItem.get("_forceItem");
      if (localForceItem != null)
      {
        localForceItem.location[0] = ((float)localVisualItem.getEndX());
        localForceItem.location[1] = ((float)localVisualItem.getEndY());
        localForceItem.force[0] = (localForceItem.force[1] = 0.0F);
        localForceItem.velocity[0] = (localForceItem.velocity[1] = 0.0F);
      }
    }
    this.m_lasttime = -1L;
  }
  
  protected void initSimulator(ForceSimulator paramForceSimulator)
  {
    TupleSet localTupleSet = this.m_vis.getGroup(this.m_nodeGroup);
    if (localTupleSet == null) {
      return;
    }
    try
    {
      localTupleSet.addColumns(FORCEITEM_SCHEMA);
    }
    catch (IllegalArgumentException localIllegalArgumentException) {}
    float f1 = this.referrer == null ? 0.0F : (float)this.referrer.getX();
    float f2 = this.referrer == null ? 0.0F : (float)this.referrer.getY();
    f1 = Float.isNaN(f1) ? 0.0F : f1;
    f2 = Float.isNaN(f2) ? 0.0F : f2;
    Iterator localIterator = this.m_vis.visibleItems(this.m_nodeGroup);
    Object localObject1;
    Object localObject2;
    while (localIterator.hasNext())
    {
      localObject1 = (VisualItem)localIterator.next();
      localObject2 = (ForceItem)((VisualItem)localObject1).get("_forceItem");
      ((ForceItem)localObject2).mass = getMassValue((VisualItem)localObject1);
      double d1 = ((VisualItem)localObject1).getEndX();
      double d2 = ((VisualItem)localObject1).getEndY();
      ((ForceItem)localObject2).location[0] = (Double.isNaN(d1) ? f1 : (float)d1);
      ((ForceItem)localObject2).location[1] = (Double.isNaN(d2) ? f2 : (float)d2);
      paramForceSimulator.addItem((ForceItem)localObject2);
    }
    if (this.m_edgeGroup != null)
    {
      localIterator = this.m_vis.visibleItems(this.m_edgeGroup);
      while (localIterator.hasNext())
      {
        localObject1 = (EdgeItem)localIterator.next();
        localObject2 = ((EdgeItem)localObject1).getSourceItem();
        ForceItem localForceItem1 = (ForceItem)((NodeItem)localObject2).get("_forceItem");
        NodeItem localNodeItem = ((EdgeItem)localObject1).getTargetItem();
        ForceItem localForceItem2 = (ForceItem)localNodeItem.get("_forceItem");
        float f3 = getSpringCoefficient((EdgeItem)localObject1);
        float f4 = getSpringLength((EdgeItem)localObject1);
        paramForceSimulator.addSpring(localForceItem1, localForceItem2, f3 >= 0.0F ? f3 : -1.0F, f4 >= 0.0F ? f4 : -1.0F);
      }
    }
  }
  
  protected float getMassValue(VisualItem paramVisualItem)
  {
    return 1.0F;
  }
  
  protected float getSpringLength(EdgeItem paramEdgeItem)
  {
    return -1.0F;
  }
  
  protected float getSpringCoefficient(EdgeItem paramEdgeItem)
  {
    return -1.0F;
  }
  
  public VisualItem getReferrer()
  {
    return this.referrer;
  }
  
  public void setReferrer(VisualItem paramVisualItem)
  {
    this.referrer = paramVisualItem;
  }
  
  static
  {
    FORCEITEM_SCHEMA.addColumn("_forceItem", ForceItem.class, new ForceItem());
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\action\layout\graph\ForceDirectedLayout.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */