package prefuse.controls;

import java.awt.Cursor;
import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;
import java.awt.geom.Point2D.Double;
import javax.swing.SwingUtilities;
import prefuse.Display;
import prefuse.Visualization;
import prefuse.data.Table;
import prefuse.data.event.TableListener;
import prefuse.visual.VisualItem;

public class DragControl
  extends ControlAdapter
  implements TableListener
{
  private VisualItem activeItem;
  protected String action;
  protected Point2D down = new Point2D.Double();
  protected Point2D temp = new Point2D.Double();
  protected boolean dragged;
  protected boolean wasFixed;
  protected boolean resetItem;
  private boolean fixOnMouseOver = true;
  protected boolean repaint = true;
  
  public DragControl() {}
  
  public DragControl(boolean paramBoolean)
  {
    this.repaint = paramBoolean;
  }
  
  public DragControl(boolean paramBoolean1, boolean paramBoolean2)
  {
    this.repaint = paramBoolean1;
    this.fixOnMouseOver = paramBoolean2;
  }
  
  public DragControl(String paramString)
  {
    this.repaint = false;
    this.action = paramString;
  }
  
  public DragControl(String paramString, boolean paramBoolean)
  {
    this.repaint = false;
    this.fixOnMouseOver = paramBoolean;
    this.action = paramString;
  }
  
  public void setFixPositionOnMouseOver(boolean paramBoolean)
  {
    this.fixOnMouseOver = paramBoolean;
  }
  
  public void itemEntered(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    Display localDisplay = (Display)paramMouseEvent.getSource();
    localDisplay.setCursor(Cursor.getPredefinedCursor(12));
    this.activeItem = paramVisualItem;
    if (this.fixOnMouseOver)
    {
      this.wasFixed = paramVisualItem.isFixed();
      this.resetItem = true;
      paramVisualItem.setFixed(true);
      paramVisualItem.getTable().addTableListener(this);
    }
  }
  
  public void itemExited(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (this.activeItem == paramVisualItem)
    {
      this.activeItem = null;
      paramVisualItem.getTable().removeTableListener(this);
      if (this.resetItem) {
        paramVisualItem.setFixed(this.wasFixed);
      }
    }
    Display localDisplay = (Display)paramMouseEvent.getSource();
    localDisplay.setCursor(Cursor.getDefaultCursor());
  }
  
  public void itemPressed(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    if (!this.fixOnMouseOver)
    {
      this.wasFixed = paramVisualItem.isFixed();
      this.resetItem = true;
      paramVisualItem.setFixed(true);
      paramVisualItem.getTable().addTableListener(this);
    }
    this.dragged = false;
    Display localDisplay = (Display)paramMouseEvent.getComponent();
    localDisplay.getAbsoluteCoordinate(paramMouseEvent.getPoint(), this.down);
  }
  
  public void itemReleased(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    if (this.dragged)
    {
      this.activeItem = null;
      paramVisualItem.getTable().removeTableListener(this);
      if (this.resetItem) {
        paramVisualItem.setFixed(this.wasFixed);
      }
      this.dragged = false;
    }
  }
  
  public void itemDragged(VisualItem paramVisualItem, MouseEvent paramMouseEvent)
  {
    if (!SwingUtilities.isLeftMouseButton(paramMouseEvent)) {
      return;
    }
    this.dragged = true;
    Display localDisplay = (Display)paramMouseEvent.getComponent();
    localDisplay.getAbsoluteCoordinate(paramMouseEvent.getPoint(), this.temp);
    double d1 = this.temp.getX() - this.down.getX();
    double d2 = this.temp.getY() - this.down.getY();
    double d3 = paramVisualItem.getX();
    double d4 = paramVisualItem.getY();
    paramVisualItem.setStartX(d3);
    paramVisualItem.setStartY(d4);
    paramVisualItem.setX(d3 + d1);
    paramVisualItem.setY(d4 + d2);
    paramVisualItem.setEndX(d3 + d1);
    paramVisualItem.setEndY(d4 + d2);
    if (this.repaint) {
      paramVisualItem.getVisualization().repaint();
    }
    this.down.setLocation(this.temp);
    if (this.action != null) {
      localDisplay.getVisualization().run(this.action);
    }
  }
  
  public void tableChanged(Table paramTable, int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    if ((this.activeItem == null) || (paramInt4 != 0) || (paramInt3 != paramTable.getColumnNumber(VisualItem.FIXED))) {
      return;
    }
    int i = this.activeItem.getRow();
    if ((i >= paramInt1) && (i <= paramInt2)) {
      this.resetItem = false;
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\controls\DragControl.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */