package prefuse.data.parser;

import java.util.Arrays;

public class ParserFactory
  implements Cloneable
{
  private static final DataParser[] DEFAULT_PARSERS = { new IntParser(), new LongParser(), new DoubleParser(), new FloatParser(), new BooleanParser(), new DateParser(), new TimeParser(), new DateTimeParser(), new StringParser() };
  private static final ParserFactory DEFAULT_FACTORY = new ParserFactory(DEFAULT_PARSERS);
  private DataParser[] m_parsers;
  private boolean[] m_isCandidate;
  
  public static ParserFactory getDefaultFactory()
  {
    return DEFAULT_FACTORY;
  }
  
  public ParserFactory()
  {
    this(DEFAULT_PARSERS);
  }
  
  public Object clone()
  {
    return new ParserFactory(this.m_parsers);
  }
  
  public ParserFactory(DataParser[] paramArrayOfDataParser)
  {
    for (int i = 0; i < paramArrayOfDataParser.length; i++) {
      if (paramArrayOfDataParser[i] == null) {
        throw new IllegalArgumentException("Input parsers must be non-null");
      }
    }
    this.m_parsers = paramArrayOfDataParser;
    this.m_isCandidate = new boolean[this.m_parsers.length];
    reset();
  }
  
  protected void reset()
  {
    Arrays.fill(this.m_isCandidate, true);
  }
  
  protected void sample(String paramString)
  {
    for (int i = 0; i < this.m_parsers.length; i++) {
      if (this.m_isCandidate[i] != 0) {
        this.m_isCandidate[i] = this.m_parsers[i].canParse(paramString);
      }
    }
  }
  
  protected DataParser getParser()
  {
    for (int i = 0; i < this.m_parsers.length; i++) {
      if (this.m_isCandidate[i] != 0) {
        return this.m_parsers[i];
      }
    }
    return null;
  }
  
  public DataParser getParser(Class paramClass)
  {
    for (int i = 0; i < this.m_parsers.length; i++) {
      if (this.m_parsers[i].getType().equals(paramClass)) {
        return this.m_parsers[i];
      }
    }
    return null;
  }
  
  public DataParser getParser(String[] paramArrayOfString, int paramInt)
  {
    return getParser(new String[][] { paramArrayOfString }, 0, paramInt);
  }
  
  public DataParser getParser(String[][] paramArrayOfString, int paramInt1, int paramInt2)
  {
    if ((paramArrayOfString == null) || (paramArrayOfString.length == 0)) {
      return null;
    }
    int i = paramArrayOfString.length;
    reset();
    for (int j = paramInt2; j < i; j++) {
      sample(paramArrayOfString[j][paramInt1]);
    }
    DataParser localDataParser = getParser();
    return localDataParser;
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\parser\ParserFactory.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */