package prefuse.data.tuple;

import java.beans.PropertyChangeListener;
import java.util.HashMap;
import java.util.Iterator;
import javax.swing.event.SwingPropertyChangeSupport;
import prefuse.data.Schema;
import prefuse.data.Table;
import prefuse.data.Tuple;
import prefuse.data.event.TupleSetListener;
import prefuse.data.expression.Expression;
import prefuse.data.expression.Predicate;
import prefuse.data.util.FilterIteratorFactory;
import prefuse.util.collections.CopyOnWriteArrayList;

public abstract class AbstractTupleSet
  implements TupleSet
{
  private CopyOnWriteArrayList m_tupleListeners;
  private HashMap m_props;
  private SwingPropertyChangeSupport m_propSupport;
  
  public Iterator tuples(Predicate paramPredicate)
  {
    if (paramPredicate == null) {
      return tuples();
    }
    return FilterIteratorFactory.tuples(this, paramPredicate);
  }
  
  public void addTupleSetListener(TupleSetListener paramTupleSetListener)
  {
    if (this.m_tupleListeners == null) {
      this.m_tupleListeners = new CopyOnWriteArrayList();
    }
    if (!this.m_tupleListeners.contains(paramTupleSetListener)) {
      this.m_tupleListeners.add(paramTupleSetListener);
    }
  }
  
  public void removeTupleSetListener(TupleSetListener paramTupleSetListener)
  {
    if (this.m_tupleListeners != null) {
      this.m_tupleListeners.remove(paramTupleSetListener);
    }
  }
  
  protected void fireTupleEvent(Table paramTable, int paramInt1, int paramInt2, int paramInt3)
  {
    if ((this.m_tupleListeners != null) && (this.m_tupleListeners.size() > 0))
    {
      Object[] arrayOfObject = this.m_tupleListeners.getArray();
      Tuple[] arrayOfTuple = new Tuple[paramInt2 - paramInt1 + 1];
      int i = 0;
      int j = paramInt1;
      while (j <= paramInt2)
      {
        arrayOfTuple[i] = paramTable.getTuple(j);
        j++;
        i++;
      }
      for (i = 0; i < arrayOfObject.length; i++)
      {
        TupleSetListener localTupleSetListener = (TupleSetListener)arrayOfObject[i];
        if (paramInt3 == 1) {
          localTupleSetListener.tupleSetChanged(this, arrayOfTuple, EMPTY_ARRAY);
        } else {
          localTupleSetListener.tupleSetChanged(this, EMPTY_ARRAY, arrayOfTuple);
        }
      }
    }
  }
  
  protected void fireTupleEvent(Tuple paramTuple, int paramInt)
  {
    if ((this.m_tupleListeners != null) && (this.m_tupleListeners.size() > 0))
    {
      Object[] arrayOfObject = this.m_tupleListeners.getArray();
      Tuple[] arrayOfTuple = { paramTuple };
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        TupleSetListener localTupleSetListener = (TupleSetListener)arrayOfObject[i];
        if (paramInt == 1) {
          localTupleSetListener.tupleSetChanged(this, arrayOfTuple, EMPTY_ARRAY);
        } else {
          localTupleSetListener.tupleSetChanged(this, EMPTY_ARRAY, arrayOfTuple);
        }
      }
    }
  }
  
  protected void fireTupleEvent(Tuple[] paramArrayOfTuple1, Tuple[] paramArrayOfTuple2)
  {
    if ((this.m_tupleListeners != null) && (this.m_tupleListeners.size() > 0))
    {
      Object[] arrayOfObject = this.m_tupleListeners.getArray();
      paramArrayOfTuple1 = paramArrayOfTuple1 == null ? EMPTY_ARRAY : paramArrayOfTuple1;
      paramArrayOfTuple2 = paramArrayOfTuple2 == null ? EMPTY_ARRAY : paramArrayOfTuple2;
      for (int i = 0; i < arrayOfObject.length; i++)
      {
        TupleSetListener localTupleSetListener = (TupleSetListener)arrayOfObject[i];
        localTupleSetListener.tupleSetChanged(this, paramArrayOfTuple1, paramArrayOfTuple2);
      }
    }
  }
  
  public boolean isAddColumnSupported()
  {
    return false;
  }
  
  public void addColumns(Schema paramSchema)
  {
    if (isAddColumnSupported()) {
      for (int i = 0; i < paramSchema.getColumnCount(); i++) {
        try
        {
          addColumn(paramSchema.getColumnName(i), paramSchema.getColumnType(i), paramSchema.getDefault(i));
        }
        catch (IllegalArgumentException localIllegalArgumentException) {}
      }
    } else {
      throw new UnsupportedOperationException();
    }
  }
  
  public void addColumn(String paramString, Class paramClass, Object paramObject)
  {
    throw new UnsupportedOperationException();
  }
  
  public void addColumn(String paramString, Class paramClass)
  {
    throw new UnsupportedOperationException();
  }
  
  public void addColumn(String paramString, Expression paramExpression)
  {
    throw new UnsupportedOperationException();
  }
  
  public void addColumn(String paramString1, String paramString2)
  {
    throw new UnsupportedOperationException();
  }
  
  public void addPropertyChangeListener(PropertyChangeListener paramPropertyChangeListener)
  {
    if (paramPropertyChangeListener == null) {
      return;
    }
    if (this.m_propSupport == null) {
      this.m_propSupport = new SwingPropertyChangeSupport(this);
    }
    this.m_propSupport.addPropertyChangeListener(paramPropertyChangeListener);
  }
  
  public void addPropertyChangeListener(String paramString, PropertyChangeListener paramPropertyChangeListener)
  {
    if (paramPropertyChangeListener == null) {
      return;
    }
    if (this.m_propSupport == null) {
      this.m_propSupport = new SwingPropertyChangeSupport(this);
    }
    this.m_propSupport.addPropertyChangeListener(paramString, paramPropertyChangeListener);
  }
  
  public void removePropertyChangeListener(PropertyChangeListener paramPropertyChangeListener)
  {
    if (paramPropertyChangeListener == null) {
      return;
    }
    if (this.m_propSupport == null) {
      return;
    }
    this.m_propSupport.removePropertyChangeListener(paramPropertyChangeListener);
  }
  
  public void removePropertyChangeListener(String paramString, PropertyChangeListener paramPropertyChangeListener)
  {
    if (paramPropertyChangeListener == null) {
      return;
    }
    if (this.m_propSupport == null) {
      return;
    }
    this.m_propSupport.removePropertyChangeListener(paramString, paramPropertyChangeListener);
  }
  
  public void putClientProperty(String paramString, Object paramObject)
  {
    Object localObject = null;
    if ((this.m_props == null) && (paramObject == null)) {
      return;
    }
    if (paramObject == null)
    {
      localObject = this.m_props.remove(paramString);
    }
    else
    {
      if (this.m_props == null) {
        this.m_props = new HashMap(2);
      }
      localObject = this.m_props.put(paramString, paramObject);
    }
    if (this.m_propSupport != null) {
      this.m_propSupport.firePropertyChange(paramString, localObject, paramObject);
    }
  }
  
  public Object getClientProperty(String paramString)
  {
    return this.m_props == null ? null : this.m_props.get(paramString);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\data\tuple\AbstractTupleSet.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */