package prefuse.render;

import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Shape;
import java.awt.geom.Line2D;
import java.awt.geom.Line2D.Double;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.Rectangle2D.Double;
import prefuse.util.ColorLib;
import prefuse.util.GraphicsLib;
import prefuse.visual.VisualItem;

public class AxisRenderer
  extends AbstractShapeRenderer
{
  private Line2D m_line = new Line2D.Double();
  private Rectangle2D m_box = new Rectangle2D.Double();
  private int m_xalign;
  private int m_yalign;
  private int m_ascent;
  
  public AxisRenderer()
  {
    this(0, 1);
  }
  
  public AxisRenderer(int paramInt1, int paramInt2)
  {
    this.m_xalign = paramInt1;
    this.m_yalign = paramInt2;
  }
  
  protected Shape getRawShape(VisualItem paramVisualItem)
  {
    double d1 = paramVisualItem.getDouble(VisualItem.X);
    double d2 = paramVisualItem.getDouble(VisualItem.Y);
    double d3 = paramVisualItem.getDouble(VisualItem.X2);
    double d4 = paramVisualItem.getDouble(VisualItem.Y2);
    this.m_line.setLine(d1, d2, d3, d4);
    if (!paramVisualItem.canGetString(VisualItem.LABEL)) {
      return this.m_line;
    }
    String str = paramVisualItem.getString(VisualItem.LABEL);
    if (str == null) {
      return this.m_line;
    }
    FontMetrics localFontMetrics = DEFAULT_GRAPHICS.getFontMetrics(paramVisualItem.getFont());
    this.m_ascent = localFontMetrics.getAscent();
    int i = localFontMetrics.getHeight();
    int j = localFontMetrics.stringWidth(str);
    double d5;
    switch (this.m_xalign)
    {
    case 2: 
      d5 = d1 + (d3 - d1) / 2.0D - j / 2;
      break;
    case 1: 
      d5 = d3 - j;
      break;
    case 0: 
    default: 
      d5 = d1;
    }
    double d6;
    switch (this.m_yalign)
    {
    case 2: 
      d6 = d2 + (d4 - d2) / 2.0D - i;
      break;
    case 0: 
      d6 = d2 - i;
      break;
    case 1: 
    default: 
      d6 = d4;
    }
    this.m_box.setFrame(d5, d6, j, i);
    return this.m_box;
  }
  
  public void render(Graphics2D paramGraphics2D, VisualItem paramVisualItem)
  {
    Shape localShape = getShape(paramVisualItem);
    GraphicsLib.paint(paramGraphics2D, paramVisualItem, this.m_line, getStroke(paramVisualItem), getRenderType(paramVisualItem));
    if (localShape == this.m_box)
    {
      float f1 = (float)this.m_box.getMinX();
      float f2 = (float)this.m_box.getMinY() + this.m_ascent;
      String str = paramVisualItem.getString(VisualItem.LABEL);
      paramGraphics2D.setFont(paramVisualItem.getFont());
      paramGraphics2D.setColor(ColorLib.getColor(paramVisualItem.getTextColor()));
      paramGraphics2D.drawString(str, f1, f2);
    }
  }
  
  public boolean locatePoint(Point2D paramPoint2D, VisualItem paramVisualItem)
  {
    Shape localShape = getShape(paramVisualItem);
    if (localShape == null) {
      return false;
    }
    if ((localShape == this.m_box) && (this.m_box.contains(paramPoint2D))) {
      return true;
    }
    double d1 = Math.max(2.0D, paramVisualItem.getSize());
    double d2 = d1 / 2.0D;
    return localShape.intersects(paramPoint2D.getX() - d2, paramPoint2D.getY() - d2, d1, d1);
  }
  
  public void setBounds(VisualItem paramVisualItem)
  {
    if (!this.m_manageBounds) {
      return;
    }
    Shape localShape = getShape(paramVisualItem);
    if (localShape == null)
    {
      paramVisualItem.setBounds(paramVisualItem.getX(), paramVisualItem.getY(), 0.0D, 0.0D);
    }
    else if (localShape == this.m_line)
    {
      GraphicsLib.setBounds(paramVisualItem, localShape, getStroke(paramVisualItem));
    }
    else
    {
      this.m_box.add(this.m_line.getX1(), this.m_line.getY1());
      this.m_box.add(this.m_line.getX2(), this.m_line.getY2());
      paramVisualItem.setBounds(this.m_box.getMinX(), this.m_box.getMinY(), this.m_box.getWidth(), this.m_box.getHeight());
    }
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\AxisRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */