package prefuse.render;

import java.awt.Shape;
import java.awt.geom.GeneralPath;
import prefuse.data.Schema;
import prefuse.util.GraphicsLib;
import prefuse.visual.VisualItem;

public class PolygonRenderer
  extends AbstractShapeRenderer
{
  public static final String POLYGON = "_polygon";
  public static final Schema POLYGON_SCHEMA = new Schema();
  private int m_polyType = 0;
  private float m_slack = 0.08F;
  private float m_epsilon = 0.1F;
  private boolean m_closed = true;
  private String m_polyfield = "_polygon";
  private GeneralPath m_path = new GeneralPath();
  
  public PolygonRenderer()
  {
    this(0);
  }
  
  public PolygonRenderer(int paramInt)
  {
    this.m_polyType = paramInt;
  }
  
  public int getPolyType()
  {
    return this.m_polyType;
  }
  
  public void setPolyType(int paramInt)
  {
    if ((paramInt < 0) || (paramInt >= 3)) {
      throw new IllegalArgumentException("Unknown edge type: " + paramInt);
    }
    this.m_polyType = paramInt;
  }
  
  public boolean isClosePath()
  {
    return this.m_closed;
  }
  
  public void setClosePath(boolean paramBoolean)
  {
    this.m_closed = paramBoolean;
  }
  
  public float getCurveSlack()
  {
    return this.m_slack;
  }
  
  public void setCurveSlack(float paramFloat)
  {
    this.m_slack = paramFloat;
  }
  
  protected Shape getRawShape(VisualItem paramVisualItem)
  {
    float[] arrayOfFloat = (float[])paramVisualItem.get(this.m_polyfield);
    if (arrayOfFloat == null) {
      return null;
    }
    float f1 = (float)paramVisualItem.getX();
    float f2 = (float)paramVisualItem.getY();
    this.m_path.reset();
    this.m_path.moveTo(f1 + arrayOfFloat[0], f2 + arrayOfFloat[1]);
    if (this.m_polyType == 0)
    {
      for (int i = 2; (i < arrayOfFloat.length) && (!Float.isNaN(arrayOfFloat[i])); i += 2) {
        this.m_path.lineTo(f1 + arrayOfFloat[i], f2 + arrayOfFloat[(i + 1)]);
      }
    }
    else
    {
      if (this.m_polyType == 1) {
        return GraphicsLib.cardinalSpline(this.m_path, arrayOfFloat, this.m_slack, this.m_closed, f1, f2);
      }
      if (this.m_polyType == 2) {
        return GraphicsLib.stackSpline(this.m_path, arrayOfFloat, this.m_epsilon, this.m_slack, this.m_closed, f1, f2);
      }
    }
    if (this.m_closed) {
      this.m_path.closePath();
    }
    return this.m_path;
  }
  
  static
  {
    POLYGON_SCHEMA.addColumn("_polygon", float[].class);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\render\PolygonRenderer.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */