package prefuse.util;

import java.awt.Color;
import prefuse.util.collections.IntObjectHashMap;

public class ColorLib
{
  private static final IntObjectHashMap colorMap = new IntObjectHashMap();
  private static int misses = 0;
  private static int lookups = 0;
  private static final float scale = 0.7F;
  public static final float[] CATEGORY_HUES = { 0.0F, 0.083333336F, 0.16666667F, 0.33333334F, 0.5F, 0.6666667F, 0.75F, 0.8333333F, 0.9166667F };
  public static final int DEFAULT_MAP_SIZE = 64;
  
  public static int rgb(int paramInt1, int paramInt2, int paramInt3)
  {
    return rgba(paramInt1, paramInt2, paramInt3, 255);
  }
  
  public static int gray(int paramInt)
  {
    return rgba(paramInt, paramInt, paramInt, 255);
  }
  
  public static int gray(int paramInt1, int paramInt2)
  {
    return rgba(paramInt1, paramInt1, paramInt1, paramInt2);
  }
  
  public static int hex(String paramString)
  {
    if (paramString.charAt(0) == '#') {
      paramString = paramString.substring(1, paramString.length());
    }
    return setAlpha(Integer.parseInt(paramString, 16), 255);
  }
  
  public static int hsb(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    return Color.HSBtoRGB(paramFloat1, paramFloat2, paramFloat3);
  }
  
  public static int hsba(float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    return setAlpha(Color.HSBtoRGB(paramFloat1, paramFloat2, paramFloat3), (int)(paramFloat4 * 255.0F + 0.5D) & 0xFF);
  }
  
  public static int rgba(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    return (paramInt4 & 0xFF) << 24 | (paramInt1 & 0xFF) << 16 | (paramInt2 & 0xFF) << 8 | (paramInt3 & 0xFF) << 0;
  }
  
  public static int rgba(float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    return ((int)(paramFloat4 * 255.0F + 0.5D) & 0xFF) << 24 | ((int)(paramFloat1 * 255.0F + 0.5D) & 0xFF) << 16 | ((int)(paramFloat2 * 255.0F + 0.5D) & 0xFF) << 8 | (int)(paramFloat3 * 255.0F + 0.5D) & 0xFF;
  }
  
  public static int color(Color paramColor)
  {
    return paramColor.getRGB();
  }
  
  public static int red(int paramInt)
  {
    return paramInt >> 16 & 0xFF;
  }
  
  public static int green(int paramInt)
  {
    return paramInt >> 8 & 0xFF;
  }
  
  public static int blue(int paramInt)
  {
    return paramInt & 0xFF;
  }
  
  public static int alpha(int paramInt)
  {
    return paramInt >> 24 & 0xFF;
  }
  
  public static int setAlpha(int paramInt1, int paramInt2)
  {
    return rgba(red(paramInt1), green(paramInt1), blue(paramInt1), paramInt2);
  }
  
  public static Color getColor(float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    return getColor(rgba(paramFloat1, paramFloat2, paramFloat3, paramFloat4));
  }
  
  public static Color getColor(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    return getColor(paramFloat1, paramFloat2, paramFloat3, 1.0F);
  }
  
  public static Color getColor(int paramInt1, int paramInt2, int paramInt3, int paramInt4)
  {
    return getColor(rgba(paramInt1, paramInt2, paramInt3, paramInt4));
  }
  
  public static Color getColor(int paramInt1, int paramInt2, int paramInt3)
  {
    return getColor(paramInt1, paramInt2, paramInt3, 255);
  }
  
  public static Color getGrayscale(int paramInt)
  {
    return getColor(paramInt, paramInt, paramInt, 255);
  }
  
  public static Color getColor(int paramInt)
  {
    Color localColor = null;
    if ((localColor = (Color)colorMap.get(paramInt)) == null)
    {
      localColor = new Color(paramInt, true);
      colorMap.put(paramInt, localColor);
      misses += 1;
    }
    lookups += 1;
    return localColor;
  }
  
  public static int getCacheMissCount()
  {
    return misses;
  }
  
  public static int getCacheLookupCount()
  {
    return lookups;
  }
  
  public static void clearCache()
  {
    colorMap.clear();
  }
  
  public static int interp(int paramInt1, int paramInt2, double paramDouble)
  {
    double d = 1.0D - paramDouble;
    return rgba((int)Math.round(paramDouble * red(paramInt2) + d * red(paramInt1)), (int)Math.round(paramDouble * green(paramInt2) + d * green(paramInt1)), (int)Math.round(paramDouble * blue(paramInt2) + d * blue(paramInt1)), (int)Math.round(paramDouble * alpha(paramInt2) + d * alpha(paramInt1)));
  }
  
  public static int darker(int paramInt)
  {
    return rgba(Math.max(0, (int)(0.7F * red(paramInt))), Math.max(0, (int)(0.7F * green(paramInt))), Math.max(0, (int)(0.7F * blue(paramInt))), alpha(paramInt));
  }
  
  public static int brighter(int paramInt)
  {
    int i = red(paramInt);
    int j = green(paramInt);
    int k = blue(paramInt);
    int m = 3;
    if ((i == 0) && (j == 0) && (k == 0)) {
      return rgba(m, m, m, alpha(paramInt));
    }
    if ((i > 0) && (i < m)) {
      i = m;
    }
    if ((j > 0) && (j < m)) {
      j = m;
    }
    if ((k > 0) && (k < m)) {
      k = m;
    }
    return rgba(Math.min(255, (int)(i / 0.7F)), Math.min(255, (int)(j / 0.7F)), Math.min(255, (int)(k / 0.7F)), alpha(paramInt));
  }
  
  public static int[] getCoolPalette(int paramInt)
  {
    int[] arrayOfInt = new int[paramInt];
    for (int i = 0; i < paramInt; i++)
    {
      float f = i / Math.max(paramInt - 1, 1.0F);
      arrayOfInt[i] = rgba(f, 1.0F - f, 1.0F, 1.0F);
    }
    return arrayOfInt;
  }
  
  public static int[] getCoolPalette()
  {
    return getCoolPalette(64);
  }
  
  public static int[] getHotPalette(int paramInt)
  {
    int[] arrayOfInt = new int[paramInt];
    for (int i = 0; i < paramInt; i++)
    {
      int j = 3 * paramInt / 8;
      float f1 = i < j ? (i + 1) / j : 1.0F;
      float f2 = i < 2 * j ? (i - j) / j : i < j ? 0.0F : 1.0F;
      float f3 = i < 2 * j ? 0.0F : (i - 2 * j) / (paramInt - 2 * j);
      arrayOfInt[i] = rgba(f1, f2, f3, 1.0F);
    }
    return arrayOfInt;
  }
  
  public static int[] getHotPalette()
  {
    return getHotPalette(64);
  }
  
  public static int[] getCategoryPalette(int paramInt, float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    int[] arrayOfInt = new int[paramInt];
    float f = paramFloat1;
    for (int i = 0; i < paramInt; i++)
    {
      int j = i % CATEGORY_HUES.length;
      if (j == 0) {
        f = paramFloat1 + i / paramInt * (paramFloat2 - paramFloat1);
      }
      arrayOfInt[i] = hsba(CATEGORY_HUES[j], f, paramFloat3, paramFloat4);
    }
    return arrayOfInt;
  }
  
  public static int[] getCategoryPalette(int paramInt)
  {
    return getCategoryPalette(paramInt, 1.0F, 0.4F, 1.0F, 1.0F);
  }
  
  public static int[] getHSBPalette(int paramInt, float paramFloat1, float paramFloat2)
  {
    int[] arrayOfInt = new int[paramInt];
    for (int i = 0; i < paramInt; i++)
    {
      float f = i / (paramInt - 1);
      arrayOfInt[i] = hsb(f, paramFloat1, paramFloat2);
    }
    return arrayOfInt;
  }
  
  public static int[] getHSBPalette()
  {
    return getHSBPalette(64, 1.0F, 1.0F);
  }
  
  public static int[] getInterpolatedPalette(int paramInt1, int paramInt2, int paramInt3)
  {
    int[] arrayOfInt = new int[paramInt1];
    for (int i = 0; i < paramInt1; i++)
    {
      float f = i / (paramInt1 - 1);
      arrayOfInt[i] = interp(paramInt2, paramInt3, f);
    }
    return arrayOfInt;
  }
  
  public static int[] getInterpolatedPalette(int paramInt1, int paramInt2)
  {
    return getInterpolatedPalette(64, paramInt1, paramInt2);
  }
  
  public static int[] getGrayscalePalette(int paramInt)
  {
    int[] arrayOfInt = new int[paramInt];
    for (int i = 0; i < paramInt; i++)
    {
      int j = Math.round(255.0F * (0.2F + 0.6F * i / (paramInt - 1)));
      arrayOfInt[(paramInt - i - 1)] = gray(j);
    }
    return arrayOfInt;
  }
  
  public static int[] getGrayscalePalette()
  {
    return getGrayscalePalette(64);
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\ColorLib.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */