package prefuse.util.force;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.Iterator;
import java.util.Random;

public class NBodyForce
  extends AbstractForce
{
  private static String[] pnames = { "GravitationalConstant", "Distance", "BarnesHutTheta" };
  public static final float DEFAULT_GRAV_CONSTANT = -1.0F;
  public static final float DEFAULT_MIN_GRAV_CONSTANT = -10.0F;
  public static final float DEFAULT_MAX_GRAV_CONSTANT = 10.0F;
  public static final float DEFAULT_DISTANCE = -1.0F;
  public static final float DEFAULT_MIN_DISTANCE = -1.0F;
  public static final float DEFAULT_MAX_DISTANCE = 500.0F;
  public static final float DEFAULT_THETA = 0.9F;
  public static final float DEFAULT_MIN_THETA = 0.0F;
  public static final float DEFAULT_MAX_THETA = 1.0F;
  public static final int GRAVITATIONAL_CONST = 0;
  public static final int MIN_DISTANCE = 1;
  public static final int BARNES_HUT_THETA = 2;
  private float xMin;
  private float xMax;
  private float yMin;
  private float yMax;
  private QuadTreeNodeFactory factory = new QuadTreeNodeFactory();
  private QuadTreeNode root;
  private Random rand = new Random(12345678L);
  
  public NBodyForce()
  {
    this(-1.0F, -1.0F, 0.9F);
  }
  
  public NBodyForce(float paramFloat1, float paramFloat2, float paramFloat3)
  {
    this.params = new float[] { paramFloat1, paramFloat2, paramFloat3 };
    this.minValues = new float[] { -10.0F, -1.0F, 0.0F };
    this.maxValues = new float[] { 10.0F, 500.0F, 1.0F };
    this.root = this.factory.getQuadTreeNode();
  }
  
  public boolean isItemForce()
  {
    return true;
  }
  
  protected String[] getParameterNames()
  {
    return pnames;
  }
  
  private void setBounds(float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    this.xMin = paramFloat1;
    this.yMin = paramFloat2;
    this.xMax = paramFloat3;
    this.yMax = paramFloat4;
  }
  
  public void clear()
  {
    clearHelper(this.root);
    this.root = this.factory.getQuadTreeNode();
  }
  
  private void clearHelper(QuadTreeNode paramQuadTreeNode)
  {
    for (int i = 0; i < paramQuadTreeNode.children.length; i++) {
      if (paramQuadTreeNode.children[i] != null) {
        clearHelper(paramQuadTreeNode.children[i]);
      }
    }
    this.factory.reclaim(paramQuadTreeNode);
  }
  
  public void init(ForceSimulator paramForceSimulator)
  {
    clear();
    float f1 = Float.MAX_VALUE;
    float f2 = Float.MAX_VALUE;
    float f3 = Float.MIN_VALUE;
    float f4 = Float.MIN_VALUE;
    Iterator localIterator = paramForceSimulator.getItems();
    while (localIterator.hasNext())
    {
      ForceItem localForceItem1 = (ForceItem)localIterator.next();
      f6 = localForceItem1.location[0];
      float f7 = localForceItem1.location[1];
      if (f6 < f1) {
        f1 = f6;
      }
      if (f7 < f2) {
        f2 = f7;
      }
      if (f6 > f3) {
        f3 = f6;
      }
      if (f7 > f4) {
        f4 = f7;
      }
    }
    float f5 = f3 - f1;
    float f6 = f4 - f2;
    if (f5 > f6) {
      f4 = f2 + f5;
    } else {
      f3 = f1 + f6;
    }
    setBounds(f1, f2, f3, f4);
    localIterator = paramForceSimulator.getItems();
    while (localIterator.hasNext())
    {
      ForceItem localForceItem2 = (ForceItem)localIterator.next();
      insert(localForceItem2);
    }
    calcMass(this.root);
  }
  
  public void insert(ForceItem paramForceItem)
  {
    try
    {
      insert(paramForceItem, this.root, this.xMin, this.yMin, this.xMax, this.yMax);
    }
    catch (StackOverflowError localStackOverflowError)
    {
      localStackOverflowError.printStackTrace();
    }
  }
  
  private void insert(ForceItem paramForceItem, QuadTreeNode paramQuadTreeNode, float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    if (paramQuadTreeNode.hasChildren) {
      insertHelper(paramForceItem, paramQuadTreeNode, paramFloat1, paramFloat2, paramFloat3, paramFloat4);
    } else if (paramQuadTreeNode.value != null)
    {
      if (isSameLocation(paramQuadTreeNode.value, paramForceItem))
      {
        insertHelper(paramForceItem, paramQuadTreeNode, paramFloat1, paramFloat2, paramFloat3, paramFloat4);
      }
      else
      {
        ForceItem localForceItem = paramQuadTreeNode.value;
        paramQuadTreeNode.value = null;
        insertHelper(localForceItem, paramQuadTreeNode, paramFloat1, paramFloat2, paramFloat3, paramFloat4);
        insertHelper(paramForceItem, paramQuadTreeNode, paramFloat1, paramFloat2, paramFloat3, paramFloat4);
      }
    }
    else {
      paramQuadTreeNode.value = paramForceItem;
    }
  }
  
  private static boolean isSameLocation(ForceItem paramForceItem1, ForceItem paramForceItem2)
  {
    float f1 = Math.abs(paramForceItem1.location[0] - paramForceItem2.location[0]);
    float f2 = Math.abs(paramForceItem1.location[1] - paramForceItem2.location[1]);
    return (f1 < 0.01D) && (f2 < 0.01D);
  }
  
  private void insertHelper(ForceItem paramForceItem, QuadTreeNode paramQuadTreeNode, float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    float f1 = paramForceItem.location[0];
    float f2 = paramForceItem.location[1];
    float f3 = (paramFloat1 + paramFloat3) / 2.0F;
    float f4 = (paramFloat2 + paramFloat4) / 2.0F;
    int i = (f1 >= f3 ? 1 : 0) + (f2 >= f4 ? 2 : 0);
    if (paramQuadTreeNode.children[i] == null)
    {
      paramQuadTreeNode.children[i] = this.factory.getQuadTreeNode();
      paramQuadTreeNode.hasChildren = true;
    }
    if ((i == 1) || (i == 3)) {
      paramFloat1 = f3;
    } else {
      paramFloat3 = f3;
    }
    if (i > 1) {
      paramFloat2 = f4;
    } else {
      paramFloat4 = f4;
    }
    insert(paramForceItem, paramQuadTreeNode.children[i], paramFloat1, paramFloat2, paramFloat3, paramFloat4);
  }
  
  private void calcMass(QuadTreeNode paramQuadTreeNode)
  {
    float f1 = 0.0F;
    float f2 = 0.0F;
    paramQuadTreeNode.mass = 0.0F;
    if (paramQuadTreeNode.hasChildren) {
      for (int i = 0; i < paramQuadTreeNode.children.length; i++) {
        if (paramQuadTreeNode.children[i] != null)
        {
          calcMass(paramQuadTreeNode.children[i]);
          paramQuadTreeNode.mass += paramQuadTreeNode.children[i].mass;
          f1 += paramQuadTreeNode.children[i].mass * paramQuadTreeNode.children[i].com[0];
          f2 += paramQuadTreeNode.children[i].mass * paramQuadTreeNode.children[i].com[1];
        }
      }
    }
    if (paramQuadTreeNode.value != null)
    {
      paramQuadTreeNode.mass += paramQuadTreeNode.value.mass;
      f1 += paramQuadTreeNode.value.mass * paramQuadTreeNode.value.location[0];
      f2 += paramQuadTreeNode.value.mass * paramQuadTreeNode.value.location[1];
    }
    paramQuadTreeNode.com[0] = (f1 / paramQuadTreeNode.mass);
    paramQuadTreeNode.com[1] = (f2 / paramQuadTreeNode.mass);
  }
  
  public void getForce(ForceItem paramForceItem)
  {
    try
    {
      forceHelper(paramForceItem, this.root, this.xMin, this.yMin, this.xMax, this.yMax);
    }
    catch (StackOverflowError localStackOverflowError)
    {
      localStackOverflowError.printStackTrace();
    }
  }
  
  private void forceHelper(ForceItem paramForceItem, QuadTreeNode paramQuadTreeNode, float paramFloat1, float paramFloat2, float paramFloat3, float paramFloat4)
  {
    float f1 = paramQuadTreeNode.com[0] - paramForceItem.location[0];
    float f2 = paramQuadTreeNode.com[1] - paramForceItem.location[1];
    float f3 = (float)Math.sqrt(f1 * f1 + f2 * f2);
    int i = 0;
    if (f3 == 0.0F)
    {
      f1 = (this.rand.nextFloat() - 0.5F) / 50.0F;
      f2 = (this.rand.nextFloat() - 0.5F) / 50.0F;
      f3 = (float)Math.sqrt(f1 * f1 + f2 * f2);
      i = 1;
    }
    int j = (this.params[1] > 0.0F) && (f3 > this.params[1]) ? 1 : 0;
    float f4;
    if (((!paramQuadTreeNode.hasChildren) && (paramQuadTreeNode.value != paramForceItem)) || ((i == 0) && ((paramFloat3 - paramFloat1) / f3 < this.params[2])))
    {
      if (j != 0) {
        return;
      }
      f4 = this.params[0] * paramForceItem.mass * paramQuadTreeNode.mass / (f3 * f3 * f3);
      paramForceItem.force[0] += f4 * f1;
      paramForceItem.force[1] += f4 * f2;
    }
    else if (paramQuadTreeNode.hasChildren)
    {
      f4 = (paramFloat1 + paramFloat3) / 2.0F;
      float f5 = (paramFloat2 + paramFloat4) / 2.0F;
      for (int k = 0; k < paramQuadTreeNode.children.length; k++) {
        if (paramQuadTreeNode.children[k] != null) {
          forceHelper(paramForceItem, paramQuadTreeNode.children[k], (k == 1) || (k == 3) ? f4 : paramFloat1, k > 1 ? f5 : paramFloat2, (k == 1) || (k == 3) ? paramFloat3 : f4, k > 1 ? paramFloat4 : f5);
        }
      }
      if (j != 0) {
        return;
      }
      if ((paramQuadTreeNode.value != null) && (paramQuadTreeNode.value != paramForceItem))
      {
        float f6 = this.params[0] * paramForceItem.mass * paramQuadTreeNode.value.mass / (f3 * f3 * f3);
        paramForceItem.force[0] += f6 * f1;
        paramForceItem.force[1] += f6 * f2;
      }
    }
  }
  
  public static final class QuadTreeNodeFactory
  {
    private int maxNodes = 50000;
    private ArrayList nodes = new ArrayList();
    
    public NBodyForce.QuadTreeNode getQuadTreeNode()
    {
      if (this.nodes.size() > 0) {
        return (NBodyForce.QuadTreeNode)this.nodes.remove(this.nodes.size() - 1);
      }
      return new NBodyForce.QuadTreeNode();
    }
    
    public void reclaim(NBodyForce.QuadTreeNode paramQuadTreeNode)
    {
      paramQuadTreeNode.mass = 0.0F;
      paramQuadTreeNode.com[0] = 0.0F;
      paramQuadTreeNode.com[1] = 0.0F;
      paramQuadTreeNode.value = null;
      paramQuadTreeNode.hasChildren = false;
      Arrays.fill(paramQuadTreeNode.children, null);
      if (this.nodes.size() < this.maxNodes) {
        this.nodes.add(paramQuadTreeNode);
      }
    }
  }
  
  public static final class QuadTreeNode
  {
    boolean hasChildren = false;
    float mass;
    float[] com = { 0.0F, 0.0F };
    ForceItem value;
    QuadTreeNode[] children = new QuadTreeNode[4];
  }
}


/* Location:              C:\Perforce\MyAsup\dev\thirdparty\lib\asup-metamodel-client-1.0.jar!\prefuse\util\force\NBodyForce.class
 * Java compiler version: 5 (49.0)
 * JD-Core Version:       0.7.1
 */